package applicationRepository;

import interfaces.ApplicationRepositoryPOA;
import interfaces.ApplicationRepositoryPackage.ApplicationNotFoundException;
import interfaces.ApplicationRepositoryPackage.ApplicationRegistrationException;
import interfaces.ApplicationRepositoryPackage.BinaryCreationException;
import interfaces.ApplicationRepositoryPackage.BinaryNotFoundException;
import interfaces.ApplicationRepositoryPackage.DirectoryCreationException;
import interfaces.ApplicationRepositoryPackage.DirectoryNotEmptyException;
import interfaces.ApplicationRepositoryPackage.DirectoryNotFoundException;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Properties;
import java.util.Vector;
import java.io.BufferedReader;
import java.lang.reflect.Array;
import java.util.StringTokenizer;


import types.AppDescription;
import types.BinaryDescription;
import types.ContentDescription;

import arsm.*;
import arsc.*;

/**
 * Created on Jun 9, 2004
 * Last changes on Feb 03, 2005
 */


/**
 * @author vidal
 */
public class ApplicationRepositoryImpl extends ApplicationRepositoryPOA {
	
	private static String rootName;
	// Begin Security
	private  ArscImpl arscApp=null; 
	private boolean secure=false;

	/**
	 * @author vidal
	 * default constructor
	 */
	public ApplicationRepositoryImpl() {
		ApplicationRepositoryImpl.rootName = "../repository_root/";
    		BufferedReader configFile = null;
      		try{
        		configFile = new BufferedReader(
                                      (new InputStreamReader
                                       (new FileInputStream(new File("apprepos.conf")))));
			String line="";
			while((line = configFile.readLine()) != null){
				StringTokenizer st = new StringTokenizer(line);
          			String configOption = st.nextToken();
          			if(configOption.equals("secureAppRepos")){
					secure=st.nextToken().contentEquals(new StringBuffer("true"));
				}

 			}
		
			
		}catch(FileNotFoundException fnfe){
          		System.err.println("AppRepos::ApplicationRepositoryImpl->>Someone asked for an inexistent file");
          		fnfe.printStackTrace();
          		System.exit(-1);
		}catch(IOException ioe){
        		ioe.printStackTrace();
        		System.exit(-1);
 		}

		// Begin Security
		if(secure)
   		{
        		arscApp=new ArscImpl();
        		arscApp.initContext();
   		}
		// End Security
	}// constructor


	/**
	 * @author vidal
	 * @param String dir_name : a name of a directory
	 * @raises DirectoryCreationException if it cannot create the directory
	 * Creates the directory with the provided dirname, if it doesn't exist
	 */ 
	public boolean createDirectory(String dirName) throws DirectoryCreationException {
		boolean create = false;
		
		if(secure)
		{
			try{
			String stringTargetId = new String(ArscImpl.getId(dirName.getBytes())); 
			stringTargetId = stringTargetId.substring(0,stringTargetId.indexOf("/"));
			dirName = new String(arscApp.checkForeignSignature(dirName.getBytes()));
			dirName = rootName + "/" + stringTargetId + "_commonDir/" +  dirName;
			}catch(Exception e)
			{
				System.err.println(e);
				return false;
			}
		}else
		{
			dirName = rootName + "/" + dirName ;
		}
		
		makeDir(dirName);
		return create;		
	}
	
	private boolean makeDir(String dirName) throws DirectoryCreationException
	{
		boolean create;
		File dir = new File(dirName);
		if (dir.equals(null)) {
			DirectoryCreationException dce =
			 	new DirectoryCreationException("couldn't create a File "+ dirName + " !");
			throw dce;
		}
		if (dir.exists() && dir.isDirectory()) {
			DirectoryCreationException dce = 
				new DirectoryCreationException("directory exists !");
			throw  dce;
		}
		create = dir.mkdirs();
		if ( !create ) {
			DirectoryCreationException dce = 
				new DirectoryCreationException("couldn't create dir.mkdirs() "+ dirName+ " !");
			throw dce;
		}
		return create;
	}


	/**
	 * @author vidal
	 * @param String dir_name: a name of a directory
	 * @raises DirectoryNotFoundException if the dirname does not exist
	 * @raises DirectoryNotEmptyException if the directory is not empty
	 * Removes the directory with the provided dirname
	 */
	public void removeDirectory(String dirName)
		throws DirectoryNotFoundException, DirectoryNotEmptyException {
		if(secure)
		{
			try{
				String stringTargetId = new String(ArscImpl.getId(dirName.getBytes())); 
				stringTargetId = stringTargetId.substring(0,stringTargetId.indexOf("/"));
				dirName = new String(arscApp.checkForeignSignature(dirName.getBytes()));
				dirName = rootName + "/" + stringTargetId + "_commonDir/" +  dirName;
				
			}catch(Exception e)
			{
				System.err.println(e);
				return;
			}
			
		}else
		{
			dirName = rootName + "/" + dirName ;
		}
		delDir(dirName);	
	}// removeDirectory method

	private void delDir(String dirName) throws DirectoryNotFoundException, DirectoryNotEmptyException 
	{
		if (dirName == null) {
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException("dirName parameter null !");
			throw dnfe;
		}
		File dirFile = new File(dirName) ;
		if ( ! dirFile.exists()) {
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
			throw dnfe;			
		}
		if (dirFile.list().length > 0) {
			DirectoryNotEmptyException dnee = 
				new  DirectoryNotEmptyException("Directory not empty !");
			throw dnee;
		}

		/* deletes the directory */
		boolean b;
		b = dirFile.delete();
		if (! b) {
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException("deleting the directory !");
			throw dnfe;
		}
	}
	
	/**
	 * @author vidal
	 * @param dir_name  the name of the directory (absolute pathname)
	 * @param app_name the name of the application (used to name the aplication description file)
	 * @raises DirectoryCreationException if it cannot create the directory
	 * @raises ApplicationRegistrationException if it cannot create the application file
	 * Registers an application in the repository. Namely, creates a directory,
	 * at the repository root, if it doesn't exist, and creates an application file (this file cannot  to exist yet)
	 */
	public void registerApplication( String dirName, String appName)
	  	throws DirectoryCreationException, ApplicationRegistrationException {

		
		if(secure)
		{
			try{
			String stringTargetId = new String(ArscImpl.getId(dirName.getBytes())); 
			stringTargetId = stringTargetId.substring(0,stringTargetId.indexOf("/"));
			dirName = new String(arscApp.checkForeignSignature(dirName.getBytes()));
			dirName = rootName + "/" + stringTargetId + "_commonDir/" +  dirName;
			appName = new String(arscApp.checkForeignSignature(appName.getBytes()));
			appName = dirName  +  appName;
			}catch(Exception e)
			{
				System.err.println(e);
			}
		}else
		{
			dirName = rootName + "/" + dirName ;
			appName = dirName + "/" + appName;
			
		}
		if (dirName == null) { 
			ApplicationRegistrationException fce = 
				new ApplicationRegistrationException("dirName parameter null !");
			throw fce;
		}	

		if (appName == null) { 
			ApplicationRegistrationException fce = 
				new ApplicationRegistrationException("appName parameter null!");
			throw fce;
		}	
		
		//TODO Synchronize
		makeDir(dirName) ;
		
		File file = new File(appName);
		if ( file.exists() && file.isFile() ) {
			ApplicationRegistrationException fce = 
				new ApplicationRegistrationException("application already exists !");
			throw fce;
		}

		try {
			/* (1) writes the application file */
			FileOutputStream appfos =new FileOutputStream(file);
			if (appfos == null) {
				ApplicationRegistrationException are = 
					new ApplicationRegistrationException("couldn't create the aplication description file !");
				throw are;
			}

			/* (2) updates the properties application file */ 
			Properties properties;
			properties = new Properties();
			
			// update the two first properties
			Integer nbinaries = new Integer(0);
			String number_of_binaries = nbinaries.toString();
			properties.setProperty("application_name",appName);
			properties.setProperty("number_of_binaries",number_of_binaries);
			properties.store(appfos,"application properties");
			appfos.close();
		}catch(IOException ioe) {
			ApplicationRegistrationException are = 
				new ApplicationRegistrationException(ioe.getMessage()
				+": writing the application description file !");
			throw are;
		}
	}// registerApplication method


	/**
	 * @author vidal
	 * @param dirname  the name of the direrctory (absolute pathname)
	 * @param app_name the name of the application
	 * @throws DirectoryNotFoundException
	 * @throws ApplicationNotFoundException 
	 * @returns the serialized application
	 * Retrieves an application description struct
	 */
	public AppDescription getAppDescription(String dirName)
			throws DirectoryNotFoundException, ApplicationNotFoundException  {
		if (dirName == null) { 
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException("dirName parameter null !");
			throw dnfe;
		}	
		// creates application description file
		System.out.println("dirName: "+ dirName);
		AppDescription appDesc = new AppDescription();
		byte targetId[] =null;
		if(secure)
		{
			targetId = ArscImpl.getId(dirName.getBytes());
			String stringTargetId = new String(targetId); 
			stringTargetId = stringTargetId.substring(0,stringTargetId.indexOf("/"));
			dirName = new String(arscApp.checkForeignSignature(dirName.getBytes()));
			dirName = rootName + "/" + stringTargetId + "_commonDir/" +  dirName;
			
		}else
		{
			dirName = rootName + "/"  + dirName;
		}
		
		appDesc.dirname=dirName;
		
		String appName = "AppDescription_appDescFile";
		
		File appFile = new File(dirName + "/" + appName);
		
		if ( !appFile.exists() || appFile.isDirectory()) {
			ApplicationNotFoundException anfe = 
				new ApplicationNotFoundException(" :: application not found !");
			throw anfe;
		}
		
		Properties properties = new Properties();
		// reads properties from input stream
		FileInputStream fis;
		appDesc.dirname = dirName;
		try {
			fis = new FileInputStream(appFile);
			properties.load(fis);
			fis.close();
			appDesc.appname = properties.getProperty("application_name");
			appDesc.numberOfBinaries = properties.getProperty("number_of_binaries");
			int nofbin = Integer.parseInt(properties.getProperty("number_of_binaries"));
			appDesc.binaryIds = new String[nofbin];

			Object[] keys = properties.keySet().toArray();
			for (int i=0, j=0; i < keys.length ;i++) {
				String key = keys[i].toString();
				if( key.substring(key.lastIndexOf("_")).equals("_binaryFile")) {
					appDesc.binaryIds[j] = properties.getProperty(key);
				  	j++;
			  	}
		  	}	
		}catch (IOException e) {
			ApplicationNotFoundException anfe = 
				new ApplicationNotFoundException(e.getMessage()
				+": reading the application description file !");
			throw anfe;
		}
		if(secure)
		{
			appDesc.appname=new String(arscApp.requestForeignSignature(targetId,appDesc.appname.getBytes()));
			appDesc.dirname=new String(arscApp.requestForeignSignature(targetId,appDesc.dirname.getBytes()));
			appDesc.numberOfBinaries=new String(arscApp.requestForeignSignature(targetId,appDesc.numberOfBinaries.getBytes()));
			for(int i=0;i<appDesc.binaryIds.length;i++)
			{
				appDesc.binaryIds[i]=new String(arscApp.requestForeignSignature(targetId,appDesc.binaryIds[i].getBytes()));
			}
		}
		return appDesc;				 
	}//getAppDescription method


	/**
	 * @author vidal
	 * @param app_Desc updated application features needed to upload a binary file
	 * @param app serialized application to be registered
	 * @raises DirectoryNotFoundException if it cannot create the directory
	 * @raises ApplicationNotFoundException if it cannot find the application
	 * @raises BinaryCreationException if it cannot create the binary code file
	 * Adds a binary code file to an application in the repository
	 */
	public void uploadApplicationBinary(BinaryDescription binaryDesc, byte[] app)
	  	throws DirectoryNotFoundException, ApplicationNotFoundException,
	  	BinaryCreationException {

		// checks the parameters
		if (binaryDesc == null || app == null) { 
			BinaryCreationException fce = 
				new BinaryCreationException("appDesc parameter null !");
			throw fce;
		}
		if(secure)
    	{	
        	app = arscApp.checkForeignSignature(app);//check ASCT signature
        	byte[] mdApp = ArscImpl.messageDigest(app); // app messagedigest
        	byte[] appDigest = new byte[app.length+mdApp.length];
        	System.arraycopy(mdApp,0,appDigest,0,mdApp.length);
        	System.arraycopy(app,0,appDigest,mdApp.length,app.length);
        	app=appDigest;
        	String stringTargetId = new String(ArscImpl.getId(binaryDesc.dirname.getBytes())); 
        	binaryDesc.dirname = new String(arscApp.checkForeignSignature(binaryDesc.dirname.getBytes()));
        	binaryDesc.appname = new String(arscApp.checkForeignSignature(binaryDesc.appname.getBytes()));
        	binaryDesc.binname = new String(arscApp.checkForeignSignature(binaryDesc.binname.getBytes()));
        	binaryDesc.platform = new String(arscApp.checkForeignSignature(binaryDesc.platform.getBytes()));
        	binaryDesc.description = new String(arscApp.checkForeignSignature(binaryDesc.description.getBytes()));
			stringTargetId = stringTargetId.substring(0,stringTargetId.indexOf("/"));
			binaryDesc.dirname = rootName + "/" + stringTargetId + "_commonDir/" +  binaryDesc.dirname;
    	}else
		{
    		binaryDesc.dirname = rootName +  binaryDesc.dirname;
		}
		
		String dirName = binaryDesc.dirname;
		String appName = dirName + "/" +binaryDesc.appname;
		String binaryPlatform = binaryDesc.platform;
		String binaryName = binaryDesc.binname;

		if (dirName == null) { 
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
			throw dnfe;
		}

		if (binaryName == null) { 
			BinaryCreationException bce = 
				new BinaryCreationException("binaryName parameter null !");
			throw bce;
		}

		// checks if directory exists
		File dirfile = new File(dirName);
		if (! dirfile.exists() ) {
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
			throw dnfe;			
		}

		// checks if application exists
		File appfile = new File(appName);
		if ( ! appfile.exists() ) {
			ApplicationNotFoundException anfe = 
				new ApplicationNotFoundException();
			throw anfe;
		}

		// checks if binary application exists
		File binaryFile = new File(dirName, binaryName) ;
		if (binaryFile.exists() && binaryFile.isFile()) {
			BinaryCreationException bce = 
				new BinaryCreationException("binary file already exists !");
			throw bce;
		}

		/* (1) writes the application binary code file */				
		FileOutputStream binaryfos;
		try {
			binaryfos = new FileOutputStream(binaryFile);
			if (binaryfos == null) {
				BinaryCreationException bce = 
					new BinaryCreationException("couldn't create the binary file !");
				throw bce;
			}
			binaryfos.write(app);
			binaryfos.close();
		} catch (IOException e) {
			BinaryCreationException bce = 
				new BinaryCreationException(e.getMessage()
				+": writing the binary code file !");
			throw bce;			
		}

		/* (2) updates the properties application file */
		Properties properties = new Properties();

		// reads properties from input stream
		FileInputStream fis;
		try {
			fis = new FileInputStream(appfile);
			properties.load(fis);
			fis.close();
		} catch (IOException e) {
			BinaryCreationException bce = 
				new BinaryCreationException(e.getMessage()
				+": writing the binary code file !");
			throw bce;			
		}
			
		// update the properties
		String number_of_binaries = properties.getProperty("number_of_binaries");
		Integer old_nbinaries = new Integer(number_of_binaries);
		Integer new_nbinaries = new Integer(old_nbinaries.intValue() + 1);
		properties.setProperty("number_of_binaries",new_nbinaries.toString());
		properties.setProperty(binaryFile.getName(), binaryPlatform);
	
		FileOutputStream fos;
		try {
			fos = new FileOutputStream(appfile);
			properties.store(fos,"application properties");
			fos.close();
		} catch (IOException e) {
			BinaryCreationException bce = 
				new BinaryCreationException(e.getMessage()
				+": writing the binary code file !");
			throw bce;			
		}
	}// uploadApplicationBinary method


	/**
	 * @author vidal
	 * Retrieves a binary code file from the application repository
	 * @param dir_name  the name of the directory (absolute pathname)
	 * @returns the serialized application
	 */
	public byte[] getApplicationBinary(String absBinaryName){
		
		//		 Begin Security
		byte[] targetId=null;
		if(secure)
	    {
			//MessagePacket psAppId = new MessagePacket(new types.Stream(absBinaryName.getBytes())); // appId unsigned
	        targetId = ArscImpl.getId(absBinaryName.getBytes()); // Target Id
	        absBinaryName = new String(arscApp.checkForeignSignature(absBinaryName.getBytes()));// check signature
	    }
		//	End Security
		
		String binary_name = absBinaryName;
		File binaryfile = new File(binary_name);
		/* reads the application binary code file */
		FileInputStream binaryfis;
		byte[] app = null;
		try {
			binaryfis = new FileInputStream(binaryfile);
			app = new byte[(int)binaryfile.length()];
			System.out.println("try-> "+ binaryfile.getName());
			binaryfis.read(app);
			binaryfis.close();
		} catch (FileNotFoundException e) {
			System.out.println(">>> FNF: "+ binaryfile.getName());
			// TODO Auto-generated catch block
			//e.printStackTrace();
		} catch (IOException e) {
			System.out.println(">>> I/O: "+ binaryfile.getName());
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(secure){
			app=arscApp.verifyDigest(app);
	        if(app!=null)
	        {	
	                app = arscApp.requestForeignSignature(targetId, app);// Request signature for id
	                return app;  // app signed        for Target
	        }else
	        {
	        		System.err.println("Message Digest Verify Fail.");
	                return new byte[1];
	        }
		}
		return app;
  	}// getApplicationBinary method


	/**
	 * @author vidal
	 * @param dirName  the name of the directory (absolute pathname)
	 * @param binName the name of the binary application
	 * @param platform the name of the platform to compose the binaryname
	 * @raises DirectoryNotFoundException if the dirname does not exist
	 * @raises ApplicationNotFoundException if the application properties file does not exist
	 * @raises BinaryNotFoundException if the binary code file does not exist
	 * Deletes an application binary stored on the ApplicationRepository
	 */
	public void deleteApplicationBinary(String dirName, String binName, String platform)
	  	throws DirectoryNotFoundException, ApplicationNotFoundException,
		BinaryNotFoundException {
		
		if(secure)
		{
			try{
			String stringTargetId = new String(ArscImpl.getId(dirName.getBytes())); 
			stringTargetId = stringTargetId.substring(0,stringTargetId.indexOf("/"));
			dirName = new String(arscApp.checkForeignSignature(dirName.getBytes()));
			dirName = rootName + "/" + stringTargetId + "_commonDir/" +  dirName;
			binName = new String(arscApp.checkForeignSignature(binName.getBytes()));
			platform = new String(arscApp.checkForeignSignature(platform.getBytes()));
			
			//appName = rootName + "/" + stringTargetId + "_commonDir/" + appName;
			}catch(Exception e)
			{
				System.err.println(e);
			}
		}else
			{
				dirName = rootName + "/" + dirName ;
			}
		
		if (dirName == null) { 
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException("dirName parameter null !");
			throw dnfe;
		}
		
		if (binName == null) { 
			ApplicationNotFoundException anfe = 
				new ApplicationNotFoundException("binaryName parameter null !");
			throw anfe;
		}
		
		if (platform == null) { 
			BinaryNotFoundException bnf = 
				new BinaryNotFoundException("platform parameter null !");
			throw bnf;
		}

		// checks if directory exists
		File dirfile = new File(dirName) ;
		if (! dirfile.exists() ) {
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
			throw dnfe;			
		}

		// checks if application description file exists
		
		String appDesc_name = dirName + "/" + "AppDescription_appDescFile";
		File appfile = new File(appDesc_name) ;
		if ( ! appfile.exists() ) {
			ApplicationNotFoundException anfe = 
				new ApplicationNotFoundException();
			throw anfe;
		}

		// checks if binary application exists
		File binaryfile = new File(dirName , binName);
		if (! binaryfile.exists()) {
			BinaryNotFoundException bnfe = 
				new BinaryNotFoundException();
			throw bnfe;
		}

		/* (1) deletes the application binary code file */
		boolean b;
		b = binaryfile.delete();
		if (! b) {
			BinaryNotFoundException bnfe = 
				new BinaryNotFoundException("deleting the binary file !");
			throw bnfe;
		}
  		
		/* (2) updates the properties application file */
		Properties properties;
		properties = new Properties();

		try {
			// reads properties from input stream
			FileInputStream fis =new FileInputStream(appfile);
			properties.load(fis);
			fis.close();

			// update the number_of_binaries property
			String number_of_binaries = new String(properties.getProperty("number_of_binaries"));
			Integer old_nbinaries = new Integer(number_of_binaries);
			Integer new_nbinaries = new Integer(old_nbinaries.intValue() - 1);
			properties.setProperty("number_of_binaries",new_nbinaries.toString());

			// remove the specified property
			if (properties.containsKey(binaryfile.getName()) ) {
				properties.remove(binaryfile.getName());
			}
				
			FileOutputStream fos = new FileOutputStream(appfile);
			properties.store(fos,"application properties");
			fos.close();
		}catch (IOException e) {
			BinaryNotFoundException bnfe = 
				new BinaryNotFoundException(e.getMessage()
				+": updating application description file !");
			throw bnfe;
		}		
	}// deleteApplicationBinary method


	/**
	 * @author vidal
	 * @param dirName  the name of the direrctory (absolute pathname)
	 * @param appName the name of the file
	 * @throws DirectoryNotFoundException if the dirname does not exist
	 * @throws ApplicationNotFoundException if the filename does not exist
	 * @throws DirectoryNotEmptyException if the directory is not empty
	 * Unregisters an application registered on the ApplicationRepository
	 */
	public void unregisterApplication (String dirName, String appName)
		throws DirectoryNotFoundException, ApplicationNotFoundException,
					 DirectoryNotEmptyException {
		if (dirName == null) { 
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException("dirName parameter null !");
			throw dnfe;
		}
		
		if(secure)
		{
			try{
			String stringTargetId = new String(ArscImpl.getId(dirName.getBytes())); 
			stringTargetId = stringTargetId.substring(0,stringTargetId.indexOf("/"));
			dirName = new String(arscApp.checkForeignSignature(dirName.getBytes()));
			dirName = rootName + "/" + stringTargetId + "_commonDir/" +  dirName;
			appName = dirName + "/" + new String(arscApp.checkForeignSignature(appName.getBytes()));
			}catch(Exception e)
			{
				System.err.println(e);
			}
		}else
		{	
			dirName = rootName + "/" + dirName ;
			appName = dirName + "/" + appName;
		}
		
		// checks if directory exists
		File dirfile = new File(dirName) ;
		if ( !dirfile.exists()) {
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
			throw dnfe;			
		}

		if (appName == null) { 
			ApplicationNotFoundException anfe = 
				new ApplicationNotFoundException("appName parameter null !");
			throw anfe;
		}	
		
		// checks if application exists
		File appfile = new File(appName) ;
		if ( (!appfile.exists()) ) {
			ApplicationNotFoundException anfe = 
				new ApplicationNotFoundException();
			throw anfe;
		}
	
		// checks if dirfile is empty, i.e., exists only the application description file
		if (dirfile.list().length > 1) {
			DirectoryNotEmptyException dnee = 
				new  DirectoryNotEmptyException("there is at least one associated binary !");
			throw dnee;
		}
		
		/* (1) delets the application file */
		boolean b;
		b = appfile.delete();
		if (! b) {
			ApplicationNotFoundException anfe = 
				new ApplicationNotFoundException("deleting the application description file !");
			throw anfe;
		}
		delDir(dirName);		
	}// unregisterApplication method


	/** TODO correct this
   	// Returns the directories and files names under the provided dirname
   	// @param dir_name the name of a directory
   	// @return an array of ContentDescription structures 
   	// @throws DirectoryNotFoundException if the dirname does not exist
   	//*/
   	public ContentDescription[] listContents(String dir_name)
		  throws DirectoryNotFoundException {
   			byte[] targetId=null;
   			File dir = null;
   			String publicDirName = rootName + "Public_commonDir";
   			if(secure)
   			{
   				targetId = ArscImpl.getId(dir_name.getBytes());
   				dir_name = new String(arscApp.checkForeignSignature(dir_name.getBytes()));
   				String stringTargetId = new String(targetId);
   				stringTargetId= stringTargetId.substring(0,stringTargetId.indexOf("/"));
   				dir_name = rootName + stringTargetId + "_commonDir" + dir_name;
   				dir = new File(dir_name);
   				if (! dir.exists()) {
   					dir.mkdir();
   					try{
   					   Runtime.getRuntime().exec("/bin/ln -s " + "../Public_commonDir" + " " +  dir_name);
   					}catch(IOException ioe){
   					   ioe.printStackTrace();
   					} 
   				}
   			}
   			else
   			{
   				dir_name = rootName + "/" + dir_name ;
   				dir = new File( dir_name);
   				if (! dir.exists()) {
   					dir.mkdir();
   				}
   			}
   			
   			if (dir_name == null) { 
				DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
				throw dnfe;
			}
					
			if (! dir.exists()) {
				DirectoryNotFoundException dnfe = 
					new DirectoryNotFoundException();
				throw dnfe;							
			}
			
			ContentDescription[] contents = (ContentDescription[] )readDirectoryContents(dir);
			if(secure)
			{ 
				String filename = null;
				String kind = null;
				for(int i =0 ; i < contents.length;i++)
				  {
					filename=contents[i].filename;
					kind = contents[i].kind;
					contents[i].filename = new String(arscApp.requestForeignSignature(targetId,filename.getBytes()));
					contents[i].kind = new String(arscApp.requestForeignSignature(targetId,kind.getBytes()));
				  }
				
			}
			return contents;	
			
	   }

   	
   	ContentDescription[] readDirectoryContents(File  dir) {
   		File[ ] contentDir = dir.listFiles();
   		int size = contentDir.length;
   		ContentDescription[] contents = new ContentDescription[size];
   		for(int i =0; i<size; i++) {
   			String name = contentDir[i].getPath();
			String kind = name.substring(name.lastIndexOf("_"));
			String relativePath = contentDir[i].getPath();
			int pos = rootName.length();
			if(secure)
			{
				pos = relativePath.indexOf("/",pos + 1);
			}else{
				pos = relativePath.indexOf("/",pos-1);
			}
				
			relativePath = relativePath.substring(pos);
   			contents[i] = new ContentDescription(relativePath,kind);
   		}
   	return  contents;
   	}
   	
   	
   	
	///*
	// Returns the number of directories and files names under the provided dirname
	// @param dir_name the name of a directory
	// @return an array of ContentDescription structures 
	// @raises DirectoryNotFoundException if the dirname does not exist
	//*/
	public int numberOfEntries(String dir_name)
		throws DirectoryNotFoundException {
		if (dir_name == null) { 
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
			throw dnfe;
		}	
			
		File dir = new File(dir_name) ;
		if (! dir.exists()) {
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
			throw dnfe;							
		}
		
		File[ ] contentDir = dir.listFiles();
		return contentDir.length;
	}
		  
		  
	/**
	 *	Tests if the provided dirname represents a directory
	 *	auxiliary method
	 *	@param dirname the name of a directory
	 * @raises DirectoryNotFoundException if the directory with the dirname provides not exists
	 * @returns true if dirname is a directory. false otherwise
	 */
	public boolean isDirectory( String dirname)
	   throws DirectoryNotFoundException {
		File dir = new File(dirname) ;
		if (! dir.exists()) {
			DirectoryNotFoundException dnfe = 
				new DirectoryNotFoundException();
			throw dnfe;							
		}
 	   return dir.isDirectory();
	 }


	/* (non-Javadoc)
	 * @see interfaces.ApplicationRepositoryOperations#getRootName()
	 */
	public String getRootName() {
		/*
		File rootFile = new File(ApplicationRepositoryImpl.rootName);
		System.out.println("root: "+rootFile.getAbsolutePath());
		return rootFile.getAbsolutePath();
		*/
		return "../repository_root/";
	} 
}


// auxiliary classes
class JustFileFilter implements FileFilter {
	//* implements the FilenameFilter.accept method
	public boolean accept(File file) {

		if (file.exists() &&file.isFile() )
			return  true;
		return false;
	}
}

class DirectoryFilter implements FileFilter {
	//* implements the FilenameFilter.accept method
	public boolean accept(File dir) {
		if (dir.exists() && dir.isDirectory() )
			return  true;
		return false;
	}
}
