extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}

#include <cassert>

#include "utils/c++/LuaUtils.hpp"
#include "utils/c++/OrbUtils.hpp"


#include "AsctStub.hpp"
#include "BspInfo.hpp"

  //----------------------------------------------------------------------
  AsctStub::AsctStub(const string & orbPath,
               const string & resourceManagementIdlPath,
               const string & asctIor){

    state = lua_open();
    OrbUtils::initLuaState(state);
    OrbUtils::loadOrb(state, orbPath);
    OrbUtils::loadIdl(state, resourceManagementIdlPath);
    OrbUtils::instantiateProxy (state, asctIor,
                                   "IDL:interfaces/Asct:1.0",
                                   "asctProxy");

  }

  //--------------------------------------------------------------------------------
  BspInfo AsctStub::registerBspNode(const string & appId, const string & bspProxyIor){

    int stackTop = lua_gettop(state);
    lua_getglobal(state, "asctProxy");
    lua_pushstring(state, "registerBspNode");
    lua_gettable(state, -2);
    lua_getglobal(state, "asctProxy");
    lua_pushstring(state, appId.c_str());
    lua_pushstring(state, bspProxyIor.c_str());
    if (lua_pcall(state, 3, 1, 0) != 0){
      cerr << "[ERROR] AsctStub::registerBspNode->Lua error: "
           << lua_tostring(state, -1) << endl;
      lua_pop(state, 1);
    }
    BspInfo bspInfo;
    lua_pushstring(state, "isProcessZero");
    lua_gettable(state, -2);
    if(lua_toboolean(state, -1) == 1)
      bspInfo.isProcessZero(true);
    else
      bspInfo.isProcessZero(false);
    lua_pop(state, 1);
    bspInfo.processZeroIor(LuaUtils::getStringFromTable(state, "processZeroIor"));
    lua_pop(state, 2);
    assert(stackTop == lua_gettop(state));
    return bspInfo;
  }
