// CkpFileStore.hpp

#ifndef CKP_STORE_HPP
#define CKP_STORE_HPP

#include <pthread.h>
#include <string>

//----------------------------------------------------------------

struct CkpData {
  class CkpStore *ckpStore;
  void *buffer;
  long nBytesTotal;
  int ckpNumber;  
};

//-------------------------------------------------------------------

class CkpStore {
  
  virtual void saveData(void *data, long nbytes, int ckpNumber);
  
public:
  
  virtual ~CkpStore(){}
  
  static void *saveData(void *data) {
    struct CkpData *ckpData = (CkpData *)data;
    (ckpData->ckpStore)->saveData(ckpData->buffer, ckpData->nBytesTotal, ckpData->ckpNumber);
    delete[] (char *)ckpData->buffer;
    delete ckpData;
    return 0;
  }

  virtual void recoverCkpData(void * & data, long & dataSize, int ckpNumber);

  void saveCkpData(void *buffer, long nBytesTotal, int ckpNumber){
    struct CkpData *ckpData = new CkpData;
    ckpData->ckpStore = this;
    ckpData->buffer = buffer;
    ckpData->nBytesTotal = nBytesTotal;
    ckpData->ckpNumber = ckpNumber;
    
    pthread_t saveThread;
    pthread_create(&saveThread, NULL,(void * (*)(void *))&CkpStore::saveData,(void *) ckpData );
    pthread_detach(saveThread);
  }
  
};

//-----------------------------------------------------------------------------

class CkpFileStore : public CkpStore {
  
  std::string path;
  int delay;

  void saveData(void *data, long nbytes, int ckpNumber);

public:

  CkpFileStore(std::string path, int delay=0) {
    this->path = path;
    this->delay = delay;
  }

  void recoverCkpData(void * & data, long & dataSize, int ckpNumber);
  
};

//-----------------------------------------------------------------------------

class CkpRepositoryStore : public CkpStore {

  struct sockaddr_in *serverAddress;
  int execNumber;  
  
  int connectToServer();
  void saveData(void *data, long nbytes, int ckpNumber);

public:
  ~CkpRepositoryStore();
  CkpRepositoryStore(std::string path, int delay=0);
  
  void recoverCkpData(void * & data, long & dataSize, int ckpNumber);
  
};

//-----------------------------------------------------------------------------

class CkpNoStore : public CkpStore {
  
  void saveData(void *data, long nbytes, int ckpNumber);
  
public:
  void recoverCkpData(void * & data, long & dataSize, int ckpNumber);
  
};

//-----------------------------------------------------------------------------

#endif // CKP_STORE_HPP
