// LrmExperiments.cpp

#include "LrmExperiments.hpp"
#include "LinuxSpecifics.hpp"

#include <iostream>
#include <sstream>
#include <fstream>
#include <unistd.h>

#include <cstdlib>
#include <sys/time.h>

class LrmExperiments *lrmExperiments = NULL;

//------------------------------------------------------------------------------

  LrmExperiments::LrmExperiments(map<string, AppInfoHolder> & launchedAppsInfo_,
                                 pthread_mutex_t & launchedAppsInfoLock_, 
                                 pthread_mutex_t & restart_mutex_) :
    launchedAppsInfo(launchedAppsInfo_), 
    launchedAppsInfoLock(launchedAppsInfoLock_),
    restart_mutex(restart_mutex_), 
    expThread (NULL)  {}

//------------------------------------------------------------------------------
    
  int LrmExperiments::prepareExperiments(string appArgs, string appNodeRequestId) {
    int newExperimentFlag = prepareExperiment (appArgs, atoi(appNodeRequestId.c_str()));

    if (newExperimentFlag == 1) {
        /* Resets data */
        nRestart = 0;
        nReschedules = 0;
        executionTimesBegin.clear();
        executionTimesEnd.clear();
    }
    else if (newExperimentFlag == 0)
        nReschedules++;
    
    return newExperimentFlag;
  }
    

  void LrmExperiments::setTimeBegin (){    
    struct timeval timeBegin; 
    gettimeofday ( &timeBegin, NULL );    
    executionTimesBegin.push_back(timeBegin);
  }

  void LrmExperiments::setTimeEnd () {    
    struct timeval timeEnd; 
    gettimeofday ( &timeEnd, NULL ); 
    executionTimesEnd.push_back(timeEnd);
  }

  int LrmExperiments::writeExperimentData (string appMainRequestId, string appId){
    
    /* Writes data to file */
    FILE * timeFile = fopen ( ("stat" + appMainRequestId + "_" + appId + ".dat").c_str() , "w" ) ;

    double meanTime = 0;
    double execTime;
    double bef, befu, aft, aftu;
    for (unsigned int i=0; i < executionTimesBegin.size(); i++) {
        bef  = ( double  ) executionTimesBegin[i].tv_sec ;
        befu = ( double  ) executionTimesBegin[i].tv_usec ;
        aft  = ( double  ) executionTimesEnd[i].tv_sec ;
        aftu = ( double  ) executionTimesEnd[i].tv_usec ;
        execTime = ( aft + aftu / 1000000. ) - ( bef + befu / 1000000. ) ;
        meanTime += execTime;
        fprintf ( timeFile , "%-8.2f | " ,  execTime ) ;
        printf  ( "%-8.2f | " ,  execTime ) ;
    }
    fprintf ( timeFile, "mean=%-8.2f ] \n" ,  meanTime/executionTimesBegin.size() ) ;
    fprintf ( timeFile, "nRestart=%-5d, nReschedule=%-5d, nTotal=%-5d \n", nRestart, nReschedules, nRestart+nReschedules);
    printf  ( "mean=%-8.2f ] \n" ,  meanTime/executionTimesBegin.size() ) ;
    printf  ( "nRestart=%-5d, nReschedule=%-5d, nTotal=%-5d \n", nRestart, nReschedules, nRestart+nReschedules); 

    fclose(timeFile);
    
    return 0;
  }

  int LrmExperiments::prepareExperiment (string args, int appNode){
    
    cerr << "---> Preparing experiment, args=" << args << endl;
    
    /** Applications with args containing '-exp<lambda>,<expNumber> '
    *   and not reinitializing */
    unsigned long startPos   = args.find("-exp");
    unsigned long restartPos = args.find("-r");
    if (startPos   == std::string::npos ||  // does not contain -exp
        restartPos != std::string::npos)    // contains -r
      return 0;
    
    /** start1: lambda
     *  start2: experiment number */
    unsigned long start1 = startPos+4;
    unsigned long end1   = args.find(',', startPos);
    unsigned long start2 = end1+1;
    unsigned long end2   = args.find(' ', end1);
            
    /** Filename is randDist<lambda>-<expNumber*4+appNode>.data */
    ostringstream fileName;
    fileName << "/home/rcamargo/randDist/data/randDist" 
         << args.substr(start1, end1 - start1) << "_" 
         << atoi( args.substr(start2, end2 - start2).c_str() ) * 6 + appNode 
         << ".data";    
    
    static string lastExperiment1;
    static string lastExperiment2;
    string currExperiment1 = args.substr(start1, end1 - start1).c_str();
    string currExperiment2 = args.substr(start2, end2 - start2).c_str();
    if (currExperiment1.compare(lastExperiment1) == 0 &&
        currExperiment2.compare(lastExperiment2) == 0)
        return 0;            
    lastExperiment1 = args.substr(start1, end1 - start1).c_str();
    lastExperiment2 = args.substr(start2, end2 - start2).c_str();
    
    /** reads data from 'fileName' */
    cerr << "---> Reading data from file: " << fileName.str() << endl;
    ifstream inFile (fileName.str().c_str());
    if (!inFile) {
        cerr << "---> Could not open File :-( " << endl;
        cerr << "---> Exiting experiment....  " << endl;
        return -1;
    }
    vector<int> failureTimesVector;
    int temp = 0;
    while (inFile.eof() == false) {
      inFile >> temp;
      failureTimesVector.push_back(temp);
    }
    
    inFile.close();
    
    cerr << "---> Data read successfully!!!! " << endl;
        
    /** Copy data to the array 'failureTimes'
     *  failureTimes[0] contains the number of stop times stored */
    unsigned int nTimes = failureTimesVector.size();
    int *failureTimes = (int *) malloc ( nTimes*sizeof(int) + 1);
    failureTimes[0] = nTimes;
    for (unsigned int i = 0; i < nTimes; i++) {
      failureTimes[i+1] = failureTimesVector[i];
    }
    
    cerr << "---> Killing old thread... " << endl;
            
    /** Kills the old experiment */
    if (expThread != NULL) {
      pthread_cancel(*expThread);
      delete expThread;
    }
    
    cerr << "---> Starting new thread... " << endl;
        
    /** Creates a new thread*/
    expThread = new pthread_t;
    pthread_create(expThread, NULL,(void * (*)(void *))
           startExperimentWrapper ,(void *) failureTimes );
    pthread_detach(*expThread);
    
    cerr << "---> Experiment SetUp!!! " << endl;
    
    return 1;
  }
      
//--------------------------------------------------------------------
  void * LrmExperiments::startExperimentWrapper(void * ptr){

    int setState = pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,   NULL);
    int setType  = pthread_setcanceltype (PTHREAD_CANCEL_DEFERRED, NULL);

    if (setType != 0 || setState != 0)
      cerr << "WARNING!!!!!! ----> Error setting thread cancelation policies." << endl;

    int *failureTimes = (int *)ptr;
    lrmExperiments->startExperiment(failureTimes);
    return NULL;

  }

  //--------------------------------------------------------------------
  void LrmExperiments::startExperiment(int *failureTimesV){
    
    vector<int> failureTimes(failureTimesV[0]);
    for (unsigned int i=0; i < failureTimes.size(); i++)
        failureTimes[i] = failureTimesV[i+1];

    delete[] failureTimesV;

    for (unsigned int i=0; i< failureTimes.size(); i++) {

        if (failureTimes[i] < 10)
            failureTimes[i] = 10;

        cerr << "LrmImpl::startExperiments -----> " 
             << "$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ Next failure in " 
             << failureTimes[i]/60 << "min and " << failureTimes[i]%60 << " sec." << endl;  
  
        pthread_testcancel();
        sleep(failureTimes[i]);
        cerr << "LrmImpl::startExperiments -----> WOKE UP!!!!!  Killing Processes, Hahahahahahaha" << endl;
        pthread_testcancel();

        /** Kills all processes running in this LRM */
        pthread_mutex_lock(&restart_mutex);
        pthread_mutex_lock(&launchedAppsInfoLock);                  
        map<string, AppInfoHolder>::iterator it = launchedAppsInfo.begin();
        for ( ; it != launchedAppsInfo.end(); it++) {
          LinuxSpecifics::killProcess((*it).second.pid());
        }

        bool allKilled = false;
        while (allKilled == false) {
            allKilled = true;
            //pthread_yield(); // Useful to wait while notify status is called          
            it = launchedAppsInfo.begin();
            for ( ; it != launchedAppsInfo.end(); it++)
                if ( LinuxSpecifics::isAppRunning((*it).first + "/" + (*it).first) )
                    allKilled = false;               
        }
        
        cerr << "LrmImpl::startExperiments -----> All processes killed! #####################" << endl;
        pthread_mutex_unlock(&launchedAppsInfoLock);            
        pthread_mutex_unlock(&restart_mutex);
      
    } // for (int i=1; i< failureTimes[0]; i++)
  }

// Start timer - usar gettimeofday
//#define LRM_BENCHMARKING_EXPERIMENTS
//#ifdef LRM_BENCHMARKING_EXPERIMENTS
//    pthread_mutex_lock(&execTimeLock);
//    ofstream experFile;
//    string ckpFileName = currentAppDir + "/ckp.conf";
//    int experId = execNumber / 1000;
//    if ( experId >= 10000 ) {
//      int experNumber = experId % 10;
//      int ckpInterval = (experId/10) % 1000;
//      if (experNumber == 0) { // No checkpoint generation
//  experFile.open(ckpFileName.c_str(), ios::out);
//  experFile << "ckpStore " << "noCheckpointing" << endl;
//      }
//      else if (experNumber == 1) { // Checkpoint generated but not saved
//  experFile.open(ckpFileName.c_str(), ios::out);
//  experFile << "ckpStore " << "ckpNoStore" << endl;
//  experFile << "interval " << ckpInterval  << endl;
//      } 
//      else if (experNumber == 2) { // Checkpoint saved to NetworkFS
//  experFile.open(ckpFileName.c_str(), ios::out);
//  experFile << "ckpStore " << "ckpLocalStore" << endl;
//  experFile << "location " << "./"              << endl;
//  experFile << "interval " << ckpInterval     << endl;
//      }
//      else if (experNumber == 3) { // Checkpoint saved to Local FS
//  experFile.open(ckpFileName.c_str(), ios::out);
//  experFile << "ckpStore " << "ckpLocalStore"                << endl;
//  experFile << "location " << localDir << execNumber << "_" << endl;
//  experFile << "interval " << ckpInterval                    << endl;
//      }
//      else if (experNumber == 4) { // Checkpoint saved to Remote Repository
//  copyFile("repos.conf", ckpFileName.c_str());
//  experFile.open(ckpFileName.c_str(), ios::app);
//  experFile << "ckpStore " << "ckpReposStore"  << endl;
//  experFile << "execId "   << execNumber      << endl;
//  experFile << "interval " << ckpInterval      << endl;
//      }
//    }
//    experFile.close();
//    //    copyFile("ckp.conf", currentAppDir + "/" + "ckp.conf");
//    pthread_mutex_unlock(&execTimeLock);
//
//#endif // LRM_BENCHMARKING_EXPERIMENTS1
    
