package asctGui;
import interfaces.ApplicationRepository;
import interfaces.ApplicationRepositoryHelper;
import interfaces.Grm;
import interfaces.GrmHelper;
import interfaces.Lrm;
import interfaces.LrmHelper;
import interfaces.ApplicationRepositoryPackage.ApplicationNotFoundException;
import interfaces.ApplicationRepositoryPackage.DirectoryNotFoundException;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;

import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

import asctGui.ExecutingApplicationsPanel;

import subtypes.ApplicationType;
import subtypes.AsctRequestId;
import subtypes.FileStruct;
import subtypes.ApplicationType;
import types.AppDescription;
import types.CommonExecutionSpecs;
import types.DistinctExecutionSpecs;
import types.OfferSpecs;

//Begin Security
import arsc.*;
import arsm.*;
import types.Stream;
//end security






  //----------------------------------------------------------------------------------
  /**
    Asct - One GUI implementation of the ASCT in Java/Swing

    @author Andrei Goldchleger
    @date February/2004

  */
  class Asct extends JFrame implements Runnable{

    //Fields---------------------------------------------------------

    private ORB orb;
    private Grm grm;
    private ApplicationRepositoryWrapperInterface appReposWrapper;
    private ApplicationRepository appRepos;
    private String grmIor;
    private String myIor;
    private long nextMainRequestId;
    private String lastSelectedDir;
    private ExecutingApplicationsPanel executingApps;
    private HashMap execAppsMap;
    //Begin Security
    private boolean secureAppRepos=false;
    ArscImpl arscAsct=null;
    // end security


	//vidal Begin -------------------------------------------------------------
	private RepositoryPanel reposPanel;
	private JPanel reposManagerPanel;
	private String appReposDir;
	private TitledBorder title;
	//vidal End-------------------------------------------------------------
		

    //-------------------------------------------------------------------------------
    Asct(){

      super("InteGrade ASCT");

      cleanup();

      //Creates a directory to hold bspExecution.conf descriptors
      File bspBir = new File("bspConfs");
      if(! bspBir.exists())
        if( ! bspBir.mkdir())
          System.err.println("Asct::Asct-->>Cannot create bspConfs dir");

      UIManager.put("Label.font",new Font("SansSerif",Font.BOLD,14));
      UIManager.put("Button.font",new Font("SansSerif",Font.BOLD,14));
      UIManager.put("RadioButton.font",new Font("SansSerif",Font.BOLD,14));
      UIManager.put("CheckBox.font",new Font("SansSerif",Font.BOLD,14));
      UIManager.put("TitledBorder.font",new Font("SansSerif",Font.BOLD,14));
      UIManager.put("List.font",new Font("SansSerif",Font.BOLD,14));
      UIManager.put("ComboBox.font",new Font("SansSerif",Font.BOLD,14));
      UIManager.put("ToolTip.font",new Font("SansSerif",Font.BOLD,14));
      UIManager.put("MenuItem.font",new Font("SansSerif",Font.BOLD,14));
      execAppsMap = new HashMap();
      lastSelectedDir = ".";
      nextMainRequestId = 0;

      //CORBA-related initialization
      initStubs();
      JToolBar toolbar = new JToolBar();
      toolbar.setFloatable (false);
      executingApps = new ExecutingApplicationsPanel("Requested Executions", this);
      
      appReposDir = appRepos.getRootName();	// vidal
      //Begin Security
      if(isSecureAppRepos())
      {
	     System.out.println("Secure Application Repository.");
        arscAsct = new ArscImpl();
        arscAsct.initContext();
        appReposWrapper = new SecureApplicationRepositoryWrapper(appRepos, arscAsct);
      }else
      {
    	  appReposWrapper = new ApplicationRepositoryWrapper(appRepos);
      }
      //end Security
      
      JPanel topPanel = new JPanel(new BorderLayout());
      topPanel.setBackground(new Color(0x000021));
      //for the new tree structure release
      ImageIcon img = new ImageIcon("images/ASCT.gif"); 
      JLabel imgLabel = new JLabel(img);
      topPanel.add(imgLabel, BorderLayout.NORTH);
      JMenuBar menuBar = new JMenuBar();
      toolbar.add(menuBar);
      setJMenuBar (menuBar);
      // Create a menu labeled Help
      JMenu help = new JMenu ("Help");
      JMenuItem item;
      // Have it call doAboutCommand when selected
      help.add (item = new JMenuItem ("About"));
      item.addActionListener (new ActionListener() {
         public void actionPerformed (ActionEvent e) {
			doAboutCommand();
		 }
      });
      menuBar.add (help);
		
      //	reposManagerPanel initialization
      reposManagerPanel = new JPanel(new BorderLayout());
      title = BorderFactory.createTitledBorder("Application Repository");
      reposManagerPanel.setBorder(title);

      //	repository panel initialization
      reposPanel = new RepositoryPanel(appReposDir, this);
      reposManagerPanel.add( reposPanel, BorderLayout.CENTER);

      //bottom panel initialization
      JPanel bottomPanel = new JPanel(new BorderLayout());
      bottomPanel.setPreferredSize(new Dimension(600,500));
      bottomPanel.add(reposManagerPanel, BorderLayout.NORTH);
      bottomPanel.add(executingApps, BorderLayout.CENTER);

      this.getContentPane().setLayout(new BorderLayout());
      this.getContentPane().add(topPanel, BorderLayout.NORTH);
      this.getContentPane().add(bottomPanel, BorderLayout.SOUTH);
      this.setIconImage(img.getImage());
      this.setLocation(new Point(400, 150));
      this.addWindowListener(new WindowAdapter(){
		 public void windowClosing(WindowEvent e){
				System.exit(0);
		 }
      });
    

      pack();
      this.setVisible(true);

    }// Asct constructor method

    //------------------------------------------------------------------
	void doAboutCommand() {
		final JDialog dialog = new JDialog (this, "About...", true);
	  	dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {dialog.dispose();}
	  	});
	  
	  	JLabel  lab1  = new JLabel ("AsctGui Version 1.0 by Andrei Goldchlerger", JLabel.CENTER);
	  	dialog.getContentPane().add (lab1, BorderLayout.NORTH);
	  	JLabel  lab2  = new JLabel ("AsctGui Version 1.0 by Alexandre Vidal", JLabel.CENTER);
	  	dialog.getContentPane().add (lab2, BorderLayout.CENTER);
	  	JButton butt = new JButton ("Close");
	  	dialog.getContentPane().add (butt, BorderLayout.SOUTH);
	  	butt.addActionListener (new ActionListener () {
			public void actionPerformed(ActionEvent e) {
		  		dialog.setVisible(false);
				dialog.setResizable(false);
		  		dialog.dispose();
			}
	  	});
	  	dialog.setSize (400, 100);
	  	dialog.setVisible(true);
	}// doAboutCommand()  method
	//-------------------------------------------------------------------------------
	


	// ***********************************************************************************
	//	action executing methods - Vidal
	// ***********************************************************************************
	/**
	 * @author vidal
	 */
	public void uploadBinary(RepositoryPanel reposPanel, String appDirName) {
		final UploadPanel upPanel = new UploadPanel(reposPanel, this, appDirName);
		final JDialog uploadDialog = new JDialog(this, "Upload Binary", true);
		upPanel.cancelButton().addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										uploadDialog.dispose();
									}	}	);
		upPanel.submitButton().addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent e) {
										upPanel.getBinary();
										uploadDialog.dispose();
									}	}	);
		uploadDialog.setContentPane(upPanel);
		uploadDialog.setLocation(new Point(250, 250));
		uploadDialog.pack();
		uploadDialog.setVisible(true);
	}

	/**
	 * @author vidal
	 */
	 public void viewDescription(AppDescription appDesc) {
	 	final DescriptionDisplayPanel descPanel = new DescriptionDisplayPanel();
		final JDialog viewDialog = new JDialog(this, "Application Description: " , true);
		viewDialog.setLocation(new Point(250, 250));
		descPanel.loadDescription(appDesc);
		viewDialog.setContentPane(descPanel);
		viewDialog.pack();
		viewDialog.setVisible(true);
	 }
	
	// ***********************************************************************************
	//	getting methods - Vidal
	// ***********************************************************************************	
	
	public ApplicationRepositoryWrapperInterface getApplicationRepositoryWrapper() {
		return appReposWrapper;
	}
       	public boolean isSecureAppRepos()
	{
		return secureAppRepos;
 	} 
	public String getApplicationRepositoryDirectory() {
		return appReposDir;
	}
	public RepositoryPanel getRepositoryPanel() {
		return reposPanel;
	}


    //-------------------------------------------------------------------------------
    // called from RegisteredApplicationPanel
    // TODO this method will be probably deprecated 
	public void requestExecution() {
		/*	AppNameHolder appName = (AppNameHolder) regApps.getSelectedItem();
		if(appName != null)
			requestExecution(appName.appPath(), appName.appId()); */
	}


    //-------------------------------------------------------------------------------
    public void requestExecution(String appPath, String appId, boolean allBinary){
      final boolean allBin = allBinary;
      final ExecuteApplicationPanel eap = new ExecuteApplicationPanel(appPath, appId);
      final JDialog execDialog = new JDialog(this, "Execute Application", true);
      eap.cancelButton().addActionListener(new ActionListener(){
                                    public void actionPerformed(ActionEvent e){
                                         execDialog.dispose();
                                       }
                                     });
      eap.submitButton().addActionListener(new ActionListener(){
                                    public void actionPerformed(ActionEvent e){
                                    	remoteExecutionRequest(eap, allBin);
                                         execDialog.dispose();
                                       }
                                     });
      execDialog.setContentPane(eap);
      execDialog.setLocation(new Point(250, 250));
      execDialog.pack();
      execDialog.setVisible(true);

    }

    /**
	 * @author Andrei
	 * @modified by Vidal
	 * execute all possible binaries if necessary. 
	 * @date 02/2005
	 */
	private void remoteExecutionRequest(ExecuteApplicationPanel eap, boolean allBinary){

      long tmpMainRequestId;
      synchronized(this){
        tmpMainRequestId = ++nextMainRequestId;
      }
      DisplayExecHolder execHolder =
                new DisplayExecHolder(eap.getAppName(), String.valueOf(tmpMainRequestId));
      synchronized(execAppsMap){
        execAppsMap.put(String.valueOf(tmpMainRequestId), execHolder);
      }


      CommonExecutionSpecs commonSpecs = new CommonExecutionSpecs();
      commonSpecs.requestingAsctIor = myIor;
      commonSpecs.grmIor = grmIor;
      commonSpecs.applicationId = eap.getAppId();
      commonSpecs.applicationConstraints = eap.getAppConstraints();
      commonSpecs.applicationPreferences = eap.getAppPreferences();

      if (allBinary) {
        // gets Ids of all available binaries
        String appDirName = eap.getAppId();
        String appDescName = appDirName + "/AppDescription_appDescFile";
        AppDescription appDesc = new AppDescription();
        try {
				appDesc = getApplicationRepositoryWrapper().getAppDescription(appDirName);
		} catch (ApplicationNotFoundException e) {
		    JOptionPane.showMessageDialog(this.getRepositoryPanel().getFrame(), e.getMessage()
		    +e.myMessage, "Exception Warning",JOptionPane.WARNING_MESSAGE);
	    } catch (DirectoryNotFoundException e) {
	        JOptionPane.showMessageDialog(this.getRepositoryPanel().getFrame(), e.getMessage()
			+e.myMessage, "Exception Warning",JOptionPane.WARNING_MESSAGE);
		}
		//TODO must be sure if exists at least one binary !!!
		commonSpecs.applicationId = eap.getAppName() // + eap.getAppId()
			+"/"+appDesc.binaryIds[0]+"_binaryFile";
		commonSpecs.appIds = appDesc.binaryIds;// all available binaries (Ids)
      }
      else {
		// assign the binaryId to only commonSpecs.appIds[] item
		String binId = eap.getAppId().substring(eap.getAppId().lastIndexOf("/")+1);		
		commonSpecs.appIds = new String[1]; 
		commonSpecs.appIds[0] = binId.substring(0,binId.lastIndexOf("_"));
		commonSpecs.applicationId = eap.getAppName()+eap.getAppId();
      }//endif
      
      	if(secureAppRepos)
      	{
    	commonSpecs.applicationId = getApplicationRepositoryWrapper().getRootName() 
    	  							+ arscAsct.id.substring(0,arscAsct.id.indexOf("/")) 
    	  							+ "_commonDir/"
    	  							+ commonSpecs.applicationId;
      	}else{
      		commonSpecs.applicationId = getApplicationRepositoryWrapper().getRootName() 
				+ commonSpecs.applicationId;
      	}
      DistinctExecutionSpecs [] distinctSpecs = null;

      if(eap.isRegular()){

        commonSpecs.appType = ApplicationType.regular;     
        distinctSpecs = new DistinctExecutionSpecs[1];
        distinctSpecs[0] = new DistinctExecutionSpecs();
        distinctSpecs[0].asctRequestId = new AsctRequestId();
        distinctSpecs[0].asctRequestId.appMainRequestId = String.valueOf(tmpMainRequestId);
        distinctSpecs[0].asctRequestId.appNodeRequestId = String.valueOf(0);
        distinctSpecs[0].applicationArgs = eap.getAppArgs();
        distinctSpecs[0].outputFiles = eap.getOutputFiles();

        RequestedExecHolder reqExecHolder = new RequestedExecHolder(eap.getInputFiles());
        execHolder.addSubRequest(String.valueOf(0), reqExecHolder);
      }
      else if(eap.isBsp()){

        String tmpBspDescriptorDir = "bspConfs" +
                                     System.getProperty("file.separator") +
                                     String.valueOf(tmpMainRequestId);

        File tmpBspBir = new File(tmpBspDescriptorDir);
        if(! tmpBspBir.exists())
            if( ! tmpBspBir.mkdir())
                System.err.println("Cannot create dir");

        String[] bspDescriptorPath = new String[eap.getTaskNum()];
        for(int i = 0; i < bspDescriptorPath.length; i++) {

            String bspDescriptorDir = 
                tmpBspDescriptorDir + System.getProperty("file.separator") + i;
            
            bspDescriptorPath[i] = bspDescriptorDir +
                                   System.getProperty("file.separator") +
                                   "bspExecution.conf";

            try{
                File bspBir = new File(bspDescriptorDir);
                if(! bspBir.exists())
                    if( ! bspBir.mkdir())
                        System.err.println("Cannot create dir");

                PrintWriter ps = new PrintWriter( new FileOutputStream(new File(bspDescriptorPath[i])));

                ps.println("appMainRequestId " + tmpMainRequestId);
                ps.println("appNodeRequestId " + i);
                ps.println("asctIor " + myIor);
                ps.println("numExecs " + eap.getTaskNum());
                ps.close();
            }
            catch(FileNotFoundException fnfe){
                fnfe.printStackTrace();
                System.exit(-1);
            }
        }
        
        commonSpecs.appType = ApplicationType.bsp;
        distinctSpecs = new DistinctExecutionSpecs[eap.getTaskNum()];
        for(int i = 0; i < distinctSpecs.length; i++)
          distinctSpecs[i] = new DistinctExecutionSpecs();
       
        for(int i = 0; i < distinctSpecs.length; i++){
          distinctSpecs[i].asctRequestId = new AsctRequestId();
          distinctSpecs[i].asctRequestId.appMainRequestId = String.valueOf(tmpMainRequestId);
          distinctSpecs[i].asctRequestId.appNodeRequestId = String.valueOf(i);
          distinctSpecs[i].applicationArgs = eap.getAppArgs();
          distinctSpecs[i].outputFiles = eap.getOutputFiles();

          String [] dstInputFiles  = new String[eap.getInputFiles().length + 1];
          System.arraycopy(eap.getInputFiles(), 0, dstInputFiles, 0, eap.getInputFiles().length);
          dstInputFiles[dstInputFiles.length - 1] = bspDescriptorPath[i];
          RequestedExecHolder reqExecHolder = new RequestedExecHolder(dstInputFiles);
          execHolder.addSubRequest(String.valueOf(i), reqExecHolder);
        }
      }
      else if(eap.isParametric()){
        commonSpecs.appType = ApplicationType.parametric;
        distinctSpecs = new DistinctExecutionSpecs[eap.getCopies().length];
        for(int i = 0; i < distinctSpecs.length; i++)
          distinctSpecs[i] = new DistinctExecutionSpecs();

        for(int i = 0; i < eap.getCopies().length; i++){
          distinctSpecs[i].asctRequestId = new AsctRequestId();
          distinctSpecs[i].asctRequestId.appMainRequestId = String.valueOf(tmpMainRequestId);
          distinctSpecs[i].asctRequestId.appNodeRequestId = String.valueOf(i);
          distinctSpecs[i].applicationArgs = eap.getCopies()[i].args();
          distinctSpecs[i].outputFiles = eap.getCopies()[i].outputFiles();
          RequestedExecHolder reqExecHolder =
                   new RequestedExecHolder(eap.getCopies()[i].inputFiles());
          execHolder.addSubRequest(String.valueOf(i), reqExecHolder);
        }
      }

      GrmStubWrapper grmStubWrapper = new GrmStubWrapper(grm);
      grmStubWrapper.remoteExecutionRequest(commonSpecs, distinctSpecs,
                                      eap.shouldForceDifferentMachines());

    }// remoteExecutionRequest method

    //-------------------------------------------------------------------------------
    public void addExecApp(OfferSpecs offerSpecs){

      DisplayExecHolder execHolder;
      synchronized(execAppsMap){
        execHolder =
           (DisplayExecHolder) execAppsMap.get(offerSpecs.asctRequestId.appMainRequestId);
      }
      execHolder.fillSubRequest(offerSpecs.asctRequestId.appNodeRequestId,
                                offerSpecs.lrmIor,
                                offerSpecs.lrmRequestId);
      if(execHolder.noPendingRequestsLeft())
        executingApps.addItem(execHolder);
    }

    //-------------------------------------------------------------------------------
    public void appFinished(AsctRequestId asctRequestId){

      DisplayExecHolder execHolder;
      synchronized(execAppsMap){
        execHolder = (DisplayExecHolder)
                         execAppsMap.get(asctRequestId.appMainRequestId);
      }                   
      if(execHolder == null)
        return;
      execHolder.nodeFinished(asctRequestId.appNodeRequestId);
      if(execHolder.allNodesFinished() && (! execHolder.collectedResults()) ){

        collectResults(asctRequestId);

        execHolder.collectedResults(true);
        executingApps.repaint();
      }
    }

    //-------------------------------------------------------------------------------
    public void collectResults(AsctRequestId asctRequestId){

      if(! (new File(asctRequestId.appMainRequestId)).mkdir()){
        System.out.println("Directory creation failed: " + asctRequestId.appMainRequestId);
        //return;
      }
      RequestedExecHolder [] execNodes;
      synchronized(execAppsMap){
        execNodes =
          ((DisplayExecHolder) execAppsMap.get(asctRequestId.appMainRequestId)).subRequests();
      }    
      System.out.println("Requesting results from " + execNodes.length + " nodes");
      for(int i = 0; i < execNodes.length; i++){
        String subpath = (asctRequestId.appMainRequestId) + "/" + i;
        if(! (new File(subpath)).mkdir()){
          System.out.println("Directory creation failed: " + subpath);
        }
        Lrm lrm = LrmHelper.narrow(orb.string_to_object(execNodes[i].lrmIor()));
        FileStruct [] returnFiles = lrm.requestOutputFiles(execNodes[i].lrmRequestId());
        for(int j = 0; j < returnFiles.length; j++){
          try{
            FileOutputStream fos =
             new FileOutputStream(subpath + "/" + returnFiles[j].filename);
            fos.write(returnFiles[j].file);
            fos.close();
          }
          catch(IOException ioe){
            System.err.println("collectResults-->> Write failed");
          }
        }//for
      }//for
    }//method

    //-------------------------------------------------------------------------------
    public void killApplication(String appMainRequestId){

      DisplayExecHolder execHolder;
      synchronized(execAppsMap){
        execHolder = (DisplayExecHolder) execAppsMap.get(appMainRequestId);
      }

      Object [] options = new Object [] {"Kill",  "Cancel"};
      int selection = JOptionPane.showOptionDialog(this,
                                   "Do you really want to kill the application: \n" +
                                   execHolder.appName() + " ?\n",
                                   "Confirm Kill",
                                   JOptionPane.YES_NO_OPTION,
                                   JOptionPane.WARNING_MESSAGE,
                                   null,//default icon
                                   options,
                                   options[1]);
      if(selection == JOptionPane.YES_OPTION){
        synchronized(execAppsMap){
          execAppsMap.remove(appMainRequestId);
        }
        executingApps.removeElement(execHolder);
        for(int i = 0; i < execHolder.subRequests().length; i++){
          Lrm lrm =
            LrmHelper.narrow(orb.string_to_object(execHolder.subRequests()[i].lrmIor()));
          lrm.kill(execHolder.subRequests()[i].lrmRequestId());
        }
      }
    }

    //-------------------------------------------------------------------------------
    public void removeApp(String appMainRequestId){

      DisplayExecHolder execHolder;
      synchronized(execAppsMap){
        execHolder = (DisplayExecHolder) execAppsMap.get(appMainRequestId);
      }

      Object [] options = new Object [] {"Remove",  "Cancel"};
      int selection = JOptionPane.showOptionDialog(this,
                                   "Do you really want to remove the execution: \n" +
                                   execHolder.appName() + " ?\n",
                                   "Confirm Remove",
                                   JOptionPane.YES_NO_OPTION,
                                   JOptionPane.WARNING_MESSAGE,
                                   null,//default icon
                                   options,
                                   options[1]);
      if(selection == JOptionPane.YES_OPTION){
        synchronized(execAppsMap){
          execAppsMap.remove(appMainRequestId);
        }
        executingApps.removeElement(execHolder);
      }
    }

    //-------------------------------------------------------------------------------
    public void viewResults(){

      DisplayExecHolder selectedExec = (DisplayExecHolder) executingApps.getSelectedItem();
      if(selectedExec == null)
        return;

      if(selectedExec == null)
        return;

      final FileViewPanel viewPanel = new FileViewPanel(selectedExec.requestId());

      final JDialog viewDialog = new JDialog(this,
                                 "View Results: " + selectedExec.appName(), true);
      viewDialog.setLocation(new Point(250, 250));
      viewDialog.setContentPane(viewPanel);
      viewDialog.pack();
      viewDialog.setVisible(true);

    }

    //-------------------------------------------------------------------------------
    private void initStubs(){
      try{
      orb = ORB.init(new String [] {}, null);
//    get a reference to the naming service
      org.omg.CORBA.Object ns = orb.resolve_initial_references("NameService");
      NamingContextExt nameService = NamingContextExtHelper.narrow( ns );
//    look up AR and GRM
      appRepos = ApplicationRepositoryHelper.narrow( nameService.resolve(nameService.to_name("AR")) );
      grm = GrmHelper.narrow( nameService.resolve(nameService.to_name("GRM")) );
      grmIor=grm.toString();
      }catch(Exception e){
    	   e.printStackTrace();    	  
      }

     BufferedReader configFile = null;
     try{
        configFile = new BufferedReader(
                    (new InputStreamReader
                        (new FileInputStream(new File("asct.conf")))));
        String line = "";
        while((line = configFile.readLine()) != null){
          StringTokenizer st = new StringTokenizer(line);
          String configOption = st.nextToken();
          if(configOption.equals("secureAppRepos"))
                secureAppRepos=st.nextToken().contentEquals(new StringBuffer("true"));
        }
      }
      catch(FileNotFoundException fnfe){
          //Difficult to happen, once that the file was choosed through a FileChooser
          System.err.println("Asct::registerApplication->>Someone asked for an inexistent file");
          fnfe.printStackTrace();
          System.exit(-1);
      }
      catch(IOException ioe){
        ioe.printStackTrace();
        System.exit(-1);
      }
	




      
	}//  initStubs() method

    //-------------------------------------------------------------------------------
    public void run(){

     try{
      POA poa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
      poa.the_POAManager().activate();
      org.omg.CORBA.Object o = poa.servant_to_reference(new AsctImpl(this));
      myIor = orb.object_to_string(o);
     }
     catch(Exception e){
      e.printStackTrace();
     }

      orb.run();

    }

    //-------------------------------------------------------------------------------
    public String  [] getAppInputFiles(AsctRequestId asctRequestId){


      DisplayExecHolder execHolder;
      synchronized(execAppsMap){
        execHolder = (DisplayExecHolder) execAppsMap.get(asctRequestId.appMainRequestId);
      }

      return execHolder.getSubRequest(asctRequestId.appNodeRequestId).inputFiles();
    }

    //-------------------------------------------------------------------------------
    private void rm(File file){
      if(file.isFile()){
        System.out.println("Deleting: " + file.getAbsolutePath());
        file.delete();
      }
      else if(file.isDirectory()){
        File [] dirContents = file.listFiles();
        for(int i = 0; i < dirContents.length; i++)
          rm(dirContents[i]);
        System.out.println("Deleting: " + file.getAbsolutePath());
        file.delete();
      }
    }
    
    //-------------------------------------------------------------------------------
    private void cleanup(){

      for(int i = 1; ; i++){
        File dir = new File(String.valueOf(i));
        if(dir.exists() && dir.isDirectory())
          rm(dir);
        else
          break;
      }

      File bspConfs = new  File("bspConfs");
      rm(bspConfs);
    }

    //-------------------------------------------------------------------------------
    public static void main(String [] args){

      Asct asct = new Asct();
      new Thread(asct).start();
    }

  }//class Asct

  //=================================================================================
  class GrmStubWrapper extends Thread{

    private Grm grm_;
    private CommonExecutionSpecs commonSpecs_;
    private DistinctExecutionSpecs [] distinctSpecs_;
    private boolean forceDifferentMachines_;

    //------------------------------------------------------------------------------
    GrmStubWrapper(Grm grm){

      grm_ = grm;

    }

    //------------------------------------------------------------------------------
    public void remoteExecutionRequest(CommonExecutionSpecs commonSpecs,
                                       DistinctExecutionSpecs [] distinctSpecs,
                                        boolean forceDifferentMachines){

      commonSpecs_ = commonSpecs;
      distinctSpecs_ = distinctSpecs;
      forceDifferentMachines_ = forceDifferentMachines;
      this.start();
    }

    //------------------------------------------------------------------------------
    public void run(){
      grm_.remoteExecutionRequest(commonSpecs_,
                                  distinctSpecs_,
                                  forceDifferentMachines_);
    }
  }//GrmStubWrapper class  

