package asctGui;
import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import java.io.FileOutputStream;

import types.OfferSpecs;
import types.BspInfo;
import subtypes.FileStruct;
import subtypes.AsctRequestId;
import interfaces.AsctPOA;

import java.util.HashMap;


  /**
    AsctImpl - Implements the ASCT interface
    
    @author Andrei Goldchleger
    @date February/2004
  */
  public class AsctImpl extends AsctPOA{

    //Fields---------------------------------------------------------------
    private Asct asct_; /**< Reference to the gui */
    private HashMap bspZerosIors_;

    //Constructor----------------------------------------------------------
    public AsctImpl(Asct asct){
      asct_ = asct;
      bspZerosIors_ = new HashMap();
    }

    //---------------------------------------------------------------------
    // /**
    //  *
    //  * Receive confirmation from a remote LRM that accepted our
    //  * execution request
    //  *
    //  * Analog to 2k's "ACK"
    //  *
    //  * @param offSpecs - Offer information, see(@see types::offSpecs)
    //  */

    public void acceptedExecutionRequest(OfferSpecs offSpecs){

        System.out.println("Our request represented by main id: " +
                            offSpecs.asctRequestId.appMainRequestId +
                           " and NODE id: " +
                            offSpecs.asctRequestId.appNodeRequestId +
                           " was ACCEPTED by LRM: " +
                            offSpecs.lrmIor);


        asct_.addExecApp(offSpecs);

    }


    //---------------------------------------------------------------------
    // /**
    //  * Receive notice that one of our queries was rejected
    //  * by all other LRMs (Though we can retry again if we
    //  * so wish)
    //  *
    //  * Analog to 2k's "NACK"
    //  *
    //  * @param requestId - Id of the refused request
    //  */
    public void refusedExecutionRequest(AsctRequestId asctRequestId){

      System.out.println("Our request represented by main id: " +
                         asctRequestId.appMainRequestId +
                         " and NODE id: " +
                         asctRequestId.appNodeRequestId +
                         " was REFUSED");

    }

    //---------------------------------------------------------------------
    // /**
    // *  Receive request for files needed to allow the remote execution of
    // *  a request made by this ASCT
    // *
    // *  @param paths -  the file path in the host where this ASCT is running
    // *  @return all files needed by the remote execution
    // */
    public FileStruct [] requestInputFiles(AsctRequestId asctRequestId){
      String [] filePaths = asct_.getAppInputFiles(asctRequestId);
      FileStruct [] inputFiles = new FileStruct[filePaths.length];
      for(int i = 0; i < inputFiles.length; i++){
        inputFiles[i] = new FileStruct();
        try{
          File file = new File(filePaths[i]);
          inputFiles[i].filename = file.getName();
          FileInputStream fis = new FileInputStream(file);
          byte [] serializedFile = new byte[(int)file.length()];//FIXME: check if cast is acceptable
          fis.read(serializedFile);
          fis.close();
          inputFiles[i].file = serializedFile;
        }
        catch(FileNotFoundException fnfe){
          System.err.println("AsctImpl::requestInputFiles-->>Someone asked for an inexistent file");
          /** @todo: Signal error condition to caller(Raise CORBA exception) */
          System.exit(-1);
          return null;
        }
        catch(IOException ioe){
          System.err.println("AsctImpl::requestInputFiles-->>IOException");
          /** @todo: Signal error condition to caller(Raise CORBA exception) */
          System.exit(-1);
          return null;
        }
      }//for
      return inputFiles;
    }//method

    //--------------------------------------------------------------------
    //FIXME: Comment me
    public void appFinished(AsctRequestId asctRequestId){ asct_.appFinished(asctRequestId); }


    //--------------------------------------------------------------------
    //FIXME: Comment me
    public BspInfo registerBspNode(String appMainRequestId, String bspProxyIor){

      synchronized(bspZerosIors_){


        if(bspZerosIors_.get(appMainRequestId) == null){

          bspZerosIors_.put(appMainRequestId, bspProxyIor);
          return new BspInfo(true, bspProxyIor);

        }
        //else
        return new BspInfo(false, (String) bspZerosIors_.get(appMainRequestId));
        //FIXME: We are not doing any cleanup here. if cleaning up, we need to register
        //in another map the number of nodes in each BSP application
      }

    }

  }//class
