package asctGui;
import java.lang.Exception;

import types.AppDescription;
import types.ContentDescription;
import interfaces.ApplicationRepository;
import arsc.*;

class SecureApplicationRepositoryWrapper extends Thread implements ApplicationRepositoryWrapperInterface{

    private ApplicationRepository appRepos_;
    private ArscImpl arsc_=null;
    SecureApplicationRepositoryWrapper(ApplicationRepository appRepos, ArscImpl arsc)
    {
      appRepos_ = appRepos;
      arsc_=arsc;
    }
public void uploadApplicationBinary(types.BinaryDescription binDesc, byte[] app)
    {
	types.BinaryDescription tmpBinDesc = new  types.BinaryDescription();
	
	try{
		app = arsc_.requestSignature(app);
		tmpBinDesc.dirname = new String(arsc_.requestSignature(binDesc.dirname.getBytes()));
		tmpBinDesc.appname = new String(arsc_.requestSignature(binDesc.appname.getBytes()));
		tmpBinDesc.binname = new String(arsc_.requestSignature(binDesc.binname.getBytes()));
		tmpBinDesc.platform = new String(arsc_.requestSignature(binDesc.platform.getBytes()));
		tmpBinDesc.description = new String(arsc_.requestSignature(binDesc.description.getBytes())); 
		appRepos_.uploadApplicationBinary(tmpBinDesc,app);
	} catch (Exception e) { 
			new Exception(e.getMessage(),e);
 	}
    }
public ContentDescription[] listContents(String dir_name)
{
    
    ContentDescription [] contents =null; 
    try{
    	dir_name = new String(arsc_.requestSignature(dir_name.getBytes()));
    	contents = appRepos_.listContents(dir_name);
    	} catch (Exception e) { 
    			new Exception(e.getMessage(),e);
     	}
    	byte[] filename = null;
    	byte[] kind = null;
    	for(int i =0 ; i < contents.length;i++)
		  {
			filename=arsc_.checkSignature(contents[i].filename.getBytes());
			kind = arsc_.checkSignature(contents[i].kind.getBytes());
			contents[i].filename = new String(filename);
			contents[i].kind = new String(kind);
		  }
    return contents;	
}

public boolean createDirectory(String dir_name) {
	 
	 try{
		 	dir_name = new String(arsc_.requestSignature(dir_name.getBytes()));
	    	return  appRepos_.createDirectory(dir_name);
	    	} catch (Exception e) { 
	    			new Exception(e.getMessage(),e);
	     	}
	    	
	return false;
	
}
public void removeDirectory(String dir_name) {
	// TODO Auto-generated method stub
	 try{
		 dir_name = new String(arsc_.requestSignature(dir_name.getBytes()));
	    	appRepos_.removeDirectory(dir_name);
	    	} catch (Exception e) { 
	    			new Exception(e.getMessage(),e);
	     	}
}
public void registerApplication(String appDirName, String appName) {
	try{
		appDirName = new String(arsc_.requestSignature(appDirName.getBytes()));
		appName = new String(arsc_.requestSignature(appName.getBytes()));
	    	appRepos_.registerApplication(appDirName, appName);
	    	} catch (Exception e) { 
	    			new Exception(e.getMessage(),e);
	     	}
	
}

public void unregisterApplication(String appDirName, String appName) {
	try{
		appDirName = new String(arsc_.requestSignature(appDirName.getBytes()));
		appName = new String(arsc_.requestSignature(appName.getBytes()));
	    	appRepos_.unregisterApplication(appDirName, appName);
	    	} catch (Exception e) { 
	    			new Exception(e.getMessage(),e);
	    			System.err.println(e);
	     	}
	
}
public void deleteApplicationBinary(String dirName, String binaryName, String plataform) {
	
	try{
		dirName = new String(arsc_.requestSignature(dirName.getBytes()));
		binaryName = new String(arsc_.requestSignature(binaryName.getBytes()));
		plataform = new String(arsc_.requestSignature(plataform.getBytes()));
	   	appRepos_.deleteApplicationBinary(dirName,binaryName,plataform);
	   	} catch (Exception e) { 
	   			new Exception(e.getMessage(),e);
	   			System.err.println(e);
	     }
}
public AppDescription getAppDescription(String dirName) {
	AppDescription appDesc=null;
	try{
		dirName = new String(arsc_.requestSignature(dirName.getBytes()));
		appDesc=appRepos_.getAppDescription(dirName);
	} catch (Exception e) { 
			new Exception(e.getMessage(),e);
			System.err.println(e);
	}
	appDesc.appname=new String(arsc_.checkSignature(appDesc.appname.getBytes()));
	appDesc.dirname=new String(arsc_.checkSignature(appDesc.dirname.getBytes()));
	appDesc.numberOfBinaries=new String(arsc_.checkSignature(appDesc.numberOfBinaries.getBytes()));
	for(int i=0;i<appDesc.binaryIds.length;i++)
	{
		appDesc.binaryIds[i]=new String(arsc_.checkSignature(appDesc.binaryIds[i].getBytes()));
	}
	return appDesc;
}
public String getRootName() {
	// TODO Auto-generated method stub
	return appRepos_.getRootName();
}


}

