package clusterView;


import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Component;
import java.awt.*;

import java.util.Set;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.ListSelectionModel;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.border.*;

import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;

import javax.swing.plaf.basic.BasicTabbedPaneUI;
import javax.swing.event.ChangeEvent;




/**
  Clusterview implements the general layout of the application. 

* @author Stefan Neusatz GuilhenStefan Neusatz Guilhen
* @version 1.0
*/
public class ClusterView extends JFrame {
    
    private JPanel northPanel, westPanel;
    private Container container;

    private JLabel refreshPeriodLabel;
    private JTextField refreshPeriodTF;
    private JButton refreshButton, historicalButton;
    private JCheckBox cpuCheckBox, ramCheckBox, diskCheckBox;
    private TitledBorder nodesBorder;
    private BorderLayout nodesLayout;
    private JScrollPane nodesScrollPane;
    private DefaultListModel nodesListModel;
    private JList nodesList;

    private final JTabbedPane tabbedPane;   // acess info frames
    private final JDesktopPane desktopPane; // access historical graphic frames

    private GridBagLayout gbLayout;
    private GridBagConstraints gbConstraints;


    // constructor
    public ClusterView() {

        this.northPanel = new JPanel();
        this.westPanel = new JPanel();
        this.refreshPeriodLabel = new JLabel();
        this.refreshPeriodTF = new JTextField();
        this.refreshButton = new JButton();
        this.nodesBorder = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,
        new Color(178, 178, 178)),"Nodes");
	this.container = getContentPane();
	this.gbLayout = new GridBagLayout();
	this.gbConstraints = new GridBagConstraints();

        this.nodesLayout = new BorderLayout();
        this.nodesScrollPane = new JScrollPane();
        this.nodesListModel = new DefaultListModel();
        this.nodesList = new JList(nodesListModel);

	this.tabbedPane = new JTabbedPane();
	this.tabbedPane.setPreferredSize(new Dimension(800,70));

	this.desktopPane = new JDesktopPane();
	this.desktopPane.setPreferredSize(new Dimension(800,120));

        try {
            clusterViewInit();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }


    
    public static void main(String[] args) {
        new Controller();
    }
    
    private void clusterViewInit() throws Exception {
        
        this.setEnabled(true);
        this.setState(Frame.NORMAL);
        this.setTitle("Cluster View");
	this.container.setLayout(gbLayout);

        // NorthPanel
        northPanel.setAlignmentY( (float) 0.5);
        northPanel.setBorder(null);
        northPanel.setOpaque(true);
        northPanel.setPreferredSize(new Dimension(30, 40));

	//        northPanel.setLayout(null);
        refreshPeriodLabel.setFont(new java.awt.Font("SansSerif", 0, 11));
        refreshPeriodLabel.setText("Refreshing Period (seconds):");
        refreshPeriodLabel.setBounds(new Rectangle(12, 12, 211, 15));
        refreshButton.setBounds(new Rectangle(756, 9, 103, 20));
        refreshButton.setFont(new java.awt.Font("SansSerif", 0, 11));
        refreshButton.setMinimumSize(new Dimension(120, 20));
        refreshButton.setPreferredSize(new Dimension(120, 20));
        refreshButton.setToolTipText("Click to refresh the view");
        refreshButton.setBorderPainted(true);
	//        refreshButton.setHorizontalAlignment(SwingConstants.EAST);
        refreshButton.setHorizontalTextPosition(SwingConstants.TRAILING);
        refreshButton.setMargin(new Insets(2, 9, 2, 9));
        refreshButton.setText("Refresh Now");
        refreshPeriodTF.setPreferredSize(new Dimension(100, 20));
        refreshPeriodTF.setBounds(new Rectangle(183, 9, 100, 20));

        northPanel.add(refreshPeriodLabel, null);
        northPanel.add(refreshPeriodTF, null);
        northPanel.add(refreshButton, null);

	//	NorthPanel grid constraints
	gbConstraints.weightx = 100;
	gbConstraints.weighty = 0;
	gbConstraints.fill = GridBagConstraints.HORIZONTAL;
        this.addComponent(northPanel,0,0,2,1);


        
        //WestPanel
        westPanel.setForeground(Color.black);
        westPanel.setPreferredSize(new Dimension(200, 50));
        westPanel.setBorder(nodesBorder);
	westPanel.setMinimumSize(new Dimension(200, 50));
	westPanel.setMaximumSize(new Dimension(200, 50));
        westPanel.setOpaque(true);
        westPanel.setLayout(nodesLayout);
        nodesBorder.setBorder(BorderFactory.createEtchedBorder());
        nodesBorder.setTitleFont(new java.awt.Font("SansSerif", 0, 11));
        nodesBorder.setTitleColor(Color.black);
        nodesList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        nodesScrollPane.getViewport().add(nodesList, null);
        westPanel.add(nodesScrollPane, BorderLayout.CENTER);

	//	WestPanel grid constraints
	gbConstraints.weightx = 0;
	gbConstraints.weighty = 100;
	gbConstraints.fill = GridBagConstraints.VERTICAL;
        this.addComponent(westPanel,1,0,1,2);


  
        // tabbedPane
        tabbedPane.setBorder(BorderFactory.createLoweredBevelBorder());
	tabbedPane.setPreferredSize(new Dimension(200, 230));
	tabbedPane.setMinimumSize(new Dimension(200, 230));
	tabbedPane.setMaximumSize(new Dimension(200, 230));

	//	tabbedPane grid constraints
	gbConstraints.weightx = 1;
	gbConstraints.weighty = 0;
	gbConstraints.fill = GridBagConstraints.HORIZONTAL;
        this.addComponent(tabbedPane,1,1,1,1);



        // desktopPane
        desktopPane.setBorder(BorderFactory.createLoweredBevelBorder());
	desktopPane.setMinimumSize(new Dimension(770, 300));
	desktopPane.setMinimumSize(new Dimension(770, 300));
	desktopPane.setMinimumSize(new Dimension(770, 300));

	//	desktopPane grid constraints
	gbConstraints.weightx = 100;
	gbConstraints.weighty = 100;
	gbConstraints.fill = GridBagConstraints.BOTH;
        this.addComponent(desktopPane,2,1,1,1);



        // MainWindow 

        this.setSize(990, 650);
	this.setVisible(true);
    }
    

    /**
    * This method updates the nodesListModel.
    * @author Stefan Neusatz Guilhen
    * @param nodes    - the names of current nodes.
    */
    public void updateNodesList(String[] nodes){
        
       String lastSelectedNode = "";
        
       if(this.nodesListModel.size() > 0 && nodesList.getSelectedIndex() != -1)
            lastSelectedNode = (String) nodesListModel.elementAt(this.nodesList.getSelectedIndex());

       this.nodesListModel.clear();
       int indexToBeSelected = -1;

       for(int i = 0; i < nodes.length; i++){
           if(nodes[i].equals(lastSelectedNode))
               indexToBeSelected = i;
           nodesListModel.addElement(nodes[i]);
       }
       this.nodesList.setSelectedIndex(indexToBeSelected);
    }


    /**
    * This method adds a component to a GridBag pane
    * @author Alexandre Csar Tavares vidal
    * @param 
    */
    private void addComponent(Component c, int row, 
			       int column, int width, int height) {
	this.gbConstraints.gridx = column;
	this.gbConstraints.gridy = row;

	this.gbConstraints.gridwidth = width;
	this.gbConstraints.gridheight = height;

	this.gbLayout.setConstraints(c, gbConstraints);
	this.container.add(c);
    }

 
    // gets and sets

    public JTextField getRefreshPeriodTF(){
        return this.refreshPeriodTF;
    }
    
    public JButton getRefreshButton(){
        return this.refreshButton;
    }
 
    public JList getNodesList(){
        return this.nodesList;
    }

    public JTabbedPane getTabbedPane(){
        return this.tabbedPane;
    }  

    public JDesktopPane getDesktopPane(){
        return this.desktopPane;
    }

}
