package clusterView;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.DefaultListModel;
import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import org.omg.CORBA.ORB;
import org.omg.CosTrading.Lookup;
import org.omg.CosTrading.LookupHelper;
import org.omg.CosTrading.Offer;
import org.omg.CosTrading.OfferIteratorHolder;
import org.omg.CosTrading.OfferSeqHolder;
import org.omg.CosTrading.Policy;
import org.omg.CosTrading.PolicyNameSeqHolder;
import org.omg.CosTrading.LookupPackage.SpecifiedProps;
import org.omg.CosTradingRepos.ServiceTypeRepository;
import org.omg.CosTradingRepos.ServiceTypeRepositoryHelper;



/**
 * The <code>Controller</code> class acts as the controller
 * component of the MVC model. It decouples the code that handle
 * the actions of the users from the view.
 *
 * @author Stefan Neusatz Guilhen
 * @see ClusterView
 * @version 1.1
 * modified by vidal 21 June 2006
 */
public class Controller {

    private long refreshPeriod;  // 
    private Timer timer;         //
    private ClusterView view;    //
    private HashMap clusterInfo; //
    private ORB orb;

    // nodes data
    private InternalFrameHandler handler; // internal frame listener
    private HashMap seleNodes;            // list of selected nodes
    private NodeFrame fNode;              // some node frame

    // historical data
    //    private HistoricalFrameHandler hHandler; // historical frame listener
    private HistoricalFrame histFrame;       // some historical frame
    private HashMap historicalFrames;        // list of historical frames
    private HashMap historicalNodes;         // list of historical nodes
    private HistoricalNode hNode;            // a historical node frame
    private int historicalNodesCount;        // historical nodes counter


    // gets and sets methods 
    public NodeFrame getFNode(String node) {
	return (NodeFrame) seleNodes.get(node);
    }
    
    // constructor
    public Controller(){
        this.refreshPeriod = 5000;

        Properties props = new Properties();
        props.put("org.omg.CORBA.ORBClass","org.jacorb.orb.ORB");
        props.put("Dorg.omg.CORBA.ORBSingletonClass","org.jacorb.orb.ORBSingleton");
        orb = ORB.init(new String [0], props);

        this.view = new ClusterView();
        this.clusterInfo = new HashMap();

        this.handler = new InternalFrameHandler();
        this.seleNodes = new HashMap();

        //	this.hHandler = new HistoricalFrameHandler();
        this.historicalNodes = new HashMap();
        this.historicalFrames = new HashMap();

        // Adds the ActionListener to the refreshPeriod Text Field
        view.getRefreshPeriodTF().addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                updateRefreshPeriod(Long.parseLong(view.
                        getRefreshPeriodTF().getText()));
                view.getRefreshPeriodTF().setText("");
            }
        });


        // adding event handlers

        // Adds the window listener to the main window. When the window closes,
        // the program is terminated.
        view.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e){
                System.exit(0);
            }
        });

        // Adds the ActionListener to the refreshButton.
        view.getRefreshButton().addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                refreshView();
            }
        });

        // Adds the MouseListener to the NodesList
        view.getNodesList().addMouseListener(new MouseAdapter(){
            public void mouseClicked(MouseEvent e){
                if (e.getClickCount() == 2){
                    
                    int index = view.getNodesList().locationToIndex(e.getPoint());
                    DefaultListModel model = (DefaultListModel)view.getNodesList().getModel();
		    //FIXME if index !null or model !empty
                    String lastSelectedNode = (String) model.elementAt(index);

		    if (!seleNodes.containsKey(lastSelectedNode)) {
			HashMap nodeInfo = new HashMap();
			nodeInfo = (HashMap)clusterInfo.get(lastSelectedNode);
			createNodeView(nodeInfo);
		    }
		}
	    }
        });


        // Bootstrap
        this.refreshView();
        
        // Starts the timer that updates the window periodically.
        timer = new Timer();
        timer.scheduleAtFixedRate(new RefreshTask(), new Date(System.currentTimeMillis()), refreshPeriod); 
    }


    /**
     * This method updates the refreshing period.
     *
     * @param period    - the refreshing period.
     */
    private void updateRefreshPeriod(long period){
        this.refreshPeriod = period;
        this.timer.cancel();
        this.timer = new Timer();
        this.timer.scheduleAtFixedRate(new RefreshTask(), new Date(), refreshPeriod*1000);
    }
      


    /**
     * This method refreshes the cluster information, by looking up in the trader.
     */
    public void refreshView() {
        
        try{
        	 Lookup lkp =
                LookupHelper.narrow(orb.resolve_initial_references("TradingService"));
            ServiceTypeRepository repos =
                ServiceTypeRepositoryHelper.narrow(lkp.type_repos());

            OfferSeqHolder oshf = new OfferSeqHolder();
            OfferIteratorHolder oih = new OfferIteratorHolder();
            PolicyNameSeqHolder pnsh = new PolicyNameSeqHolder();

            String[] desired_prop_names = {"hostName", "osName", "osVersion", "processorName",
                                           "processorMhz", "totalRAM", "totalSwap", "fsTotal",
                                           "filesTotal", "freeRAM", "freeSwap", "fsFree",
                                           "filesFree", "cpuUsage", "isSelected"};

            SpecifiedProps desired_props = new SpecifiedProps();
            desired_props.prop_names(desired_prop_names);
            
            lkp.query("NodeInfo",
            "" , //return all offers
            "", //preferences, in this case, none
            new Policy[] {},//default policies
            desired_props,0, oshf, oih, pnsh);//FIXME: Maybe it is possible to use NULL as unused parameters
            
            OfferSeqHolder osh = new OfferSeqHolder();
            //if(oih == null) System.out.println("null oih");
            //if(oih.value == null) System.out.println("null oih.value");
            
            //To avoid problems when ClusterView is initiated on an empty trader
            if(oih.value != null){ 
		//System.out.println("oih.value.max_left = " +oih.value.max_left()); BUG:always 1 
                osh.value = new Offer[oih.value.max_left()];
                oih.value.next_n(oih.value.max_left(),osh);
		//System.out.println("#de nos:"+osh.value.length); BUG:always 1
                for(int i = 0; i < osh.value.length; i++){
                    HashMap specs = new HashMap();
                    String nodeName = null;
                    for(int j =0; j < osh.value[i].properties.length; j++){
                        
                        if(osh.value[i].properties[j].name.equals("totalRAM")    ||
                            osh.value[i].properties[j].name.equals("totalSwap")  ||
                            osh.value[i].properties[j].name.equals("fsTotal")    ||
                            osh.value[i].properties[j].name.equals("filesTotal") ||
                            osh.value[i].properties[j].name.equals("freeRAM")    ||
                            osh.value[i].properties[j].name.equals("freeSwap")   ||
                            osh.value[i].properties[j].name.equals("fsFree")     ||
                            osh.value[i].properties[j].name.equals("filesFree")  ||
                            osh.value[i].properties[j].name.equals("processorMhz")
                        )
                        
                        specs.put(osh.value[i].properties[j].name,
                            new Integer(osh.value[i].properties[j].value.extract_long()));
                        else
                            specs.put(osh.value[i].properties[j].name,
                        osh.value[i].properties[j].value.extract_string());
                        if(osh.value[i].properties[j].name.equals("hostName"))
                            nodeName = osh.value[i].properties[j].value.extract_string();
                    }
		    
                    this.clusterInfo.put(nodeName,specs);
                }
            }
            

            Set set = this.clusterInfo.keySet();
            String[] nodeNames = new String[set.size()];
            set.toArray(nodeNames);

	    this.view.updateNodesList(nodeNames);

	    updateNodesFrames();
	    updateHistoricalData();
	    updateHistoricalFrames();

        } catch(Exception e){
            e.printStackTrace();
        }
        
    }


    /********* node information frame manipulation methods *************/

    /**
     * This method "creates a nodeFrame for a selected node, yet not created"
     * It is called by a mouse click event handler when an user select an
     * item at the nodes list by a double click on it.
     * @author Alexandre Csar Tavares Vidal
     */     
    public void createNodeView(HashMap nodeProperties) {
	String selectedNode = (String) nodeProperties.get("hostName");
	NodeFrame frame = new NodeFrame(selectedNode, this, false, true, false, false);
	frame.updateInternalPanel(nodeProperties);
	this.view.getTabbedPane().addTab(selectedNode,frame);
	seleNodes.put(selectedNode,frame);
	frame.addInternalFrameListener(handler);
    }


   /**
     * This method "updates information for currently opened node frames" and
     * it's callled by the "refreshView" method. That method is called either 
     * periodically, by timer events, or  by the "refresh now" button event 
     * handler when an user press the "refresh now" button. 
     * @author Alexandre Csar Tavares Vidal
     */
    public void updateNodesFrames() {

	if (!seleNodes.isEmpty()) {

	    Set set = this.seleNodes.keySet();
            String[] names = new String[set.size()];
            set.toArray(names);

	    for(int i = 0; i < seleNodes.size(); i++) {
		fNode = (NodeFrame)seleNodes.get(names[i]);
		if (clusterInfo.containsKey(names[i]))
		    fNode.updateInternalPanel((HashMap)clusterInfo.get(names[i]));
		else {
		    try {
			fNode.setClosed(true); }
		    catch (Exception e) {
			e.printStackTrace();
		    }
		}
	    }
	}
    }



   /************** historical data manipulation methods ******************/

   /**
     * This method "updates historical nodes information for all nodes", and
     * it's callled by the "refreshView" method. That method is called either 
     * periodically, by timer events, or  by the "refresh now" button event 
     * handler when an user press the "refresh now" button.
     * @author Alexandre Csar Tavares Vidal
     */
     public void updateHistoricalData() {

	if (!clusterInfo.isEmpty()) {
	    Set set = this.clusterInfo.keySet();
            String[] names = new String[set.size()];
            set.toArray(names);
	    for(int i = 0; i < clusterInfo.size(); i++) {
		if (!historicalNodes.containsKey(names[i]))
		    this.hNode = new HistoricalNode(this);
		else
		    this.hNode = (HistoricalNode)historicalNodes.get(names[i]);
		hNode.updateHistoricalNode((HashMap)clusterInfo.get(names[i]));
		this.historicalNodes.put(names[i],this.hNode);
	    }
	}
    }



   /************ historical graphics frames manipulation methods ***********/

   /**
     * This method updates "historical charts for currently opened 
     * historical views", and it's callled by the "refreshView" method. 
     * That method is called either periodically, by timer events, or  
     * by the "refresh now" button event handler when an user press 
     * the "refresh now" button.
     * @author Alexandre Csar Tavares Vidal
     */
    public void updateHistoricalFrames() {

	JInternalFrame [] allFrames = this.view.getDesktopPane().getAllFrames();
	if ( allFrames.length != 0 ) {
	    for(int i = 0; i < allFrames.length; i++) {
		this.histFrame = (HistoricalFrame)allFrames[i];
		this.hNode = (HistoricalNode)historicalNodes.get(allFrames[i].getTitle());
		this.histFrame.updateHistoricalPanel(this.hNode); 
	    }
	}
    }


    /**
     * This method "updates a historicalFrame for a selected node". 
     * It is called by an ItemEvent handler when an user
     * click on a "JCheckBox" at a specific node frame.
     * @author Alexandre Csar Tavares Vidal
     */
    public void updateOneHistoricalFrame(String nodeName) {
	boolean criar = true;
	JInternalFrame[] allFrames = this.view.getDesktopPane().getAllFrames();
	for(int i = 0; i < allFrames.length; i++) {
	    if (allFrames[i].getTitle() == nodeName) {
		this.hNode = (HistoricalNode)historicalNodes.get(nodeName);
		this.histFrame.updateHistoricalPanel(this.hNode);
		criar = false;
		i = allFrames.length;
	    }
	}
	if (criar)
	    createHistFrame(nodeName);
    }


    /**
     * This method "creates a historicalFrame for a selected node". It's
     * called by the  updateOneHistoricalFrame(String nodeName) method.
     * @author Alexandre Csar Tavares Vidal
     */
    public void createHistFrame(String nodeName) {
	    this.histFrame = new HistoricalFrame(nodeName, this, false, true, false,false);
	    //	    this.histFrame.addInternalFrameListener(hHandler);
	    this.view.getDesktopPane().add(this.histFrame);
	    this.hNode = (HistoricalNode)historicalNodes.get(nodeName);
	    this.histFrame.updateHistoricalPanel(this.hNode);
	    moveHistFrameToFront(this.histFrame);
    }
 

    /**
     * This method "moves a historicalFrame to front".
     * @author Alexandre Csar Tavares Vidal
     */
    public void moveHistFrameToFront(JInternalFrame frame) {
		this.view.getDesktopPane().moveToFront(frame);
    }



    /********************** utilities methods ****************************/

    public int roundPercentage(HashMap specs, String partField, String totalField){
        float total = ((Integer) specs.get(totalField)).floatValue();
        float part = ((Integer) specs.get(partField)).floatValue();
        //We want the usage percentage. So, subtract
        return (int) Math.round(100 *(1 - (part / total)) );
    }




    /********************* Inner Classes for event handlers ***************/
 
    /**
     * Inner class that represents a task that is invoked by the timer.
     * This class calls the <code>refreshView</code> method of the view in 
     * order to refresh the components of the view.
     * @author  Stefan Neusatz Guilhen
     */
    class RefreshTask extends TimerTask{
        /**
         * The action to be performed by this timer task.
         */
        public void run(){
            refreshView();
        }
    }


   /**
     * Inner class that implements an InternalFrameListener
     * @author  Alexandre Csar Tavares Vidal
     */
    class InternalFrameHandler extends InternalFrameAdapter  {

        /**
         * The action to be performed  when a InternalFrame (node
	 * information frame) is closing.
         */
        public void internalFrameClosing(InternalFrameEvent e) {
	    String nodeName = e.getInternalFrame().getTitle();

	    // close historical frame window if exists
	    JInternalFrame[] allHistFrames = view.getDesktopPane().getAllFrames();
	    for(int i = 0; i < allHistFrames.length; i++)	
		if (allHistFrames[i].getTitle() == nodeName) {
		    try {
			allHistFrames[i].setClosed(true);
			i = allHistFrames.length;
		    } catch(Exception event) {
			event.printStackTrace();
		    }
		}
	    seleNodes.remove(nodeName);
	}


	/*
	 * The action to be performed when a InternalFrame (node 
	 * information frame) is  activated.
	 */
        public void internalFrameActivated(InternalFrameEvent e) {
	    System.out.println("activeted 1 = ");
	    /*
	    String name = e.getInternalFrame().getTitle();
	    JInternalFrame[] allFrames = view.getDesktopPane().getAllFrames();
	    for(int i = 0; i < allFrames.length; i++){
		allFrames[i].setVisible(false);	
		if (allFrames[i].getTitle()== name) {
		    System.out.println("activeted 2 = " + name);
		    allFrames[i].show();
		    i = allFrames.length; 
		} 
	    }
	    */
	}

    }

    /*
     * Inner class that acts like a ChangeListener
     * @author  Alexandre Csar Tavares Vidal
     *
    class TabbedPaneHandler extends BasicTabbedPaneUI.TabSelectionHandler  {

	/*
	 * The action to be performed when a TabbedPane is selected.
	 *
	 public void internalFrameClosing(InternalFrameEvent e) {
	    System.out.println("activeted 1 = " + name);
	    /*	     String nodeName = e.getInternalFrame().getTitle();
	     historicalFrames.remove(nodeName);
	     }*
	
    }
    */

}
