package clusterView;

import java.util.Date;
import java.util.HashMap;
import java.util.Properties;
import java.util.Set;
import java.util.LinkedList;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import javax.swing.JInternalFrame;
import javax.swing.DefaultListModel;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.WindowConstants;


/**
 * The <code>Historical</code> class acts as a model
 * component of the MVC model. It holds historical information regarding
 * the machines in an InteGrade cluster.
 * @author Alexandre Csar Tavares Vidal
 * @version 1.0
 */

public class HistoricalNode {

    private String nodeName;
    private LinkedList nodeProperties;
    private Controller myManager;
    private Usage usage;

    // constructor
    public HistoricalNode(Controller ctrl) {
	myManager = ctrl;
	this.nodeProperties = new LinkedList();
    }



    /**
     * This method updates a historicalNode
     */
    public void updateHistoricalNode(HashMap specs) {
	int cpu, ram, disk;
	nodeName = (String)specs.get("hostName");

	if ( specs.get("cpuUsage") != null ) {
	    cpu = (int) Math.round(Float.parseFloat((String)specs.get("cpuUsage")));
	    ram = myManager.roundPercentage(specs, "freeRAM", "totalRAM");
	    disk = myManager.roundPercentage(specs, "fsFree", "fsTotal");
	} else {
	    cpu = 0;
	    ram = 0;
	    disk = 0;
	}
	usage = new Usage(cpu,ram,disk);

	if ( nodeProperties.size() == 100 ) {
	    nodeProperties.removeFirst();
	    nodeProperties.addLast(usage);
	} else
	    this.nodeProperties.add(usage);
    }

	
    // gets and sets
    public LinkedList getNodeProperties() {
	return nodeProperties;
    }

    public String getNodeName() {
	return nodeName;
    }

}



/**
* Class containing historical data 
* for one historicalNode at one instant
* @author  Alexandre Csar Tavares Vidal
*/
class Usage {
    private int cpuUsage;
    private int ramUsage;
    private int diskUsage;


    // constructors
    public Usage(int cpu, int ram, int disk){
	cpuUsage = cpu;
	ramUsage = ram;
	diskUsage = disk;
    }

    public Usage(){
	cpuUsage = 0;
	ramUsage = 0;
	diskUsage = 0;
    }

    public int getCpuUsage() {
	return cpuUsage;
    }

    public int getRamUsage() {
	return ramUsage;
    }

    public int getDiskUsage() {
	return diskUsage;
    }
}
